import QtQuick 2.12
import "../qml"

/* Data Table Element
* Show a series of ListModels as columns of data in a Table
*/

Item {
    id: root

    // REQUIRED property: for translation to work (via qsTranslate), it must know where the external model is stored.
    // Without this you will likely see blank strings in your data table.
    property string sourceFile: "DataTable"

    // default column width, used if column model has no defined width
    property int colWidth: 360 * main.hScaleFactor

    /* list of ListModels, one for each column of the table
    // properties may be attached to each model (as shown in demoModel below), to control the following:
    //      heading - text for header)
    //      hAlign - horizontal alignment for data inside cells
    //      width - width of column
    //      pointSize: point size of the text in the cell
    //      showLines: show horizontal solid lines below each row
    //      showDottedLines: show horizontal dotted lines below each row

    // The "label" role of each ListElement is used as the data in each cell in the column.
    // The "cellFillColor" role of each element may be used to override the model property of the same name for a particular cell
    // Each model in the list is another column in the table.
    */

    property list<ListModel> models

    // sample ListMode data
    ListModel
    {
        id: demoModel
        property string heading: qsTr("DEMO COLUMN") + application.emptyString
        property int width: 300
        property int hAlign: Text.AlignHCenter
        property bool showLines: true
        property bool showDottedLines: true
        property bool showFill: false

        ListElement { label: QT_TR_NOOP("ELAPSED TIME") ; halign: Text.AlignLeft }
        ListElement { label: QT_TR_NOOP("AVG HEART RATE") }
        ListElement { label: QT_TR_NOOP("MAX HEART RATE") }
        ListElement { label: QT_TR_NOOP("AVERAGE PACE (RPM)") }
        ListElement { label: QT_TR_NOOP("MAX PACE (RPM)") }
        ListElement { label: QT_TR_NOOP("DISTANCE") }
        ListElement { label: QT_TR_NOOP("CALORIES") }
    }

    Component {
        id: entryDelegate
        Item {
            id: entryItem
            property variant modelObj: ListView.view.model
            property int parentIndex: ListView.view.parentIndex
            property bool lastEntry: index === ListView.view.count-1

            width: parent.width
            height: (root.height) / (ListView.view.count + 1)

            Text {
                id: entryText
                text: {
                    if (!label) {
                        return ""
                    }

                    if (typeof label === 'string') {
                        var str
                        if (model.metricLabel && workoutScreen.unitState === "METRIC")
                            str = qsTranslate(root.sourceFile, metricLabel)
                        else
                            str = qsTranslate(root.sourceFile, label)
                        if (model.metricUnits && model.usUnits)
                            str += " (" + qsTranslate(root.sourceFile, workoutScreen.unitState === "METRIC" ? metricUnits : usUnits) + ")"
                        return str + application.emptyString
                    } else {
                        return label
                    }
                }
                font.pointSize: parent.modelObj.hasOwnProperty("pointSize") ? (parent.modelObj.pointSize * main.vScaleFactor) : (14 * main.vScaleFactor)
                font.bold: false
                font.family: parent.modelObj.hasOwnProperty("isNumeric") ? ( parent.modelObj.isNumeric ? "Free Sans" : "Montserrat Semi Bold" ) : "Montserrat Semi Bold"
                // choose color of text in cell from priority: cellTextColor property from ListElement, cellTextColor propery from ListModel property, or default black
                color: main.colorFontLight //(typeof cellTextColor !== 'undefined') ? cellTextColor : (parent.modelObj.hasOwnProperty("cellTextColor") ? parent.modelObj.cellTextColor : "black")
                width: parent.width
                height: parent.height
                horizontalAlignment: parent.modelObj.hAlign
                verticalAlignment: Text.AlignVCenter
            }

            Rectangle {
                id: rule
                visible: entryItem.modelObj.hasOwnProperty("showLines") ? entryItem.modelObj.showLines : false
                width: entryItem.width
                height: 2 * main.vScaleFactor
                color: main.colorUIBorder
                anchors { top: entryText.bottom ; left: parent.left }
            }

            Row{
                id: ruleDots
                visible: entryItem.modelObj.hasOwnProperty("showDottedLines") ? !lastEntry && entryItem.modelObj.showDottedLines : false
                width: entryItem.width
                height: dotSize
                spacing: dotSize
                anchors { top: entryText.bottom ; left: parent.left }

                property int dotSize: 3*main.scaleFactor
                property int numDots: width/ruleDots.dotSize

                Repeater {
                    model: ruleDots.numDots
                    Rectangle {
                        color: main.colorUIBorder
                        width: ruleDots.dotSize
                        height: ruleDots.dotSize
                        radius: ruleDots.dotSize/2
                        z: entryItem.z + 1
                    }
                }
            }

            Rectangle {
                id: fillbg
                visible: parent.modelObj.hasOwnProperty("showFill") ? parent.modelObj.showFill : false
                anchors.fill: parent
                color: (typeof cellFillColor !== 'undefined') ? cellFillColor : (parent.modelObj.hasOwnProperty("cellFillColor") ? parent.modelObj.cellFillColor : main.colorDialogBackground)
                z: entryText.z - 1
            }
        }
    } // end Component entryDelegate

    Component {
        id: colview

        Item {
            id: columnItem

            property variant model

            width: model ? model.hasOwnProperty("width") ? model.width : root.colWidth : root.colWidth
            height: root.height //500
            z: index === 0 ? 10 : 0

            ScalableImage {
                id: verticalRule
                property bool showEdgeRule: columnItem.model ? columnItem.model.hasOwnProperty("showEdgeRule") ? true : false : false
                anchors {
                    top: columnItem.top
                    left: columnItem.left
                }

                height: columnItem.height
                width: 5 * main.hScaleFactor

                fillMode: Image.TileVertically
                horizontalAlignment: Image.AlignHCenter
                verticalAlignment: Image.AlignTop

                sourceName: "Components/dotted_line_separator_single"
                visible: showEdgeRule
                z: colListView.z + 1
            }

            // column header text
            Text {
                id: headingText
                text:  columnItem.model ? qsTranslate(root.sourceFile, columnItem.model.heading) : ""
                color: columnItem.model ? columnItem.model.hasOwnProperty("headingTextColor") ? columnItem.model.cellTextColor : "white" : "white"
                font.bold: false
                font.pointSize: columnItem.model && columnItem.model.hasOwnProperty("headingPointSize") ? (columnItem.model.headingPointSize * main.vScaleFactor) : (16 * main.vScaleFactor)
                width: parent.width
                height: (root.height) / (colListView.count + 1)
                horizontalAlignment: Text.AlignHCenter
                verticalAlignment: Text.AlignVCenter
                anchors{ top: parent.top; topMargin: 0; left: parent.left ; leftMargin: 0;}
            }

            Rectangle {
                id: ruleHeader
                visible: columnItem.model && columnItem.model.hasOwnProperty("showDottedLines") ? parent.model.showDottedLines : false
                width: columnItem.width
                height: 2 * main.vScaleFactor
                color: main.colorUIBorder
                anchors { top: headingText.bottom ; left: columnItem.left }
                z: colListView.z + 1
            }

            ListView {
                id: colListView
                property int hAlign: columnItem.model ? columnItem.model.hAlign : Text.AlignHCenter
                property int parentIndex: index
                interactive: false
                clip: true

                width: parent.width
                height: parent.height
                delegate: entryDelegate
                anchors{ top: headingText.bottom; topMargin: 0; left: parent.left ; leftMargin: 0 }
                model: columnItem.model
            }
        } // end Item top
    } // end Component colview

    // row of table columns
    Row {
        spacing: 0
        Repeater {
            model: root.models.length
            delegate: colview

            onItemAdded: { // assign a model from the array to each column
                item.model = root.models[index]
            }
        } // end Repeater
    } // end Row
}

