import QtQuick 2.12
//~ import QtQuick.Controls 2.12

Item {
    id: root

    /**
     * Begin collapsed (default state)
     */
    property bool expanded: false
    state: expanded ? "expanded" : "collapsed"
    
    /**
     * Flag to determine if clicking causes expanding
     */
    property bool clickToExpand: true

    /**
      * Offset expanded box from collapsed box box
      */
    property real marginOffsetX: 10 * main.hScaleFactor
    property real marginOffsetY: -5 * main.vScaleFactor

    /**
     * Time in ms for animations
     */
    property int  expandTime: 1000
    
    /**
     * Dimensions of boxes when collapsed and expanded
     */
    readonly property real collapsedWidth:  collapsedContent.width + marginOffsetX
    readonly property real collapsedHeight: collapsedContent.height
    readonly property real  expandedWidth:   expandedContent.width + marginOffsetX
    readonly property real  expandedHeight:  collapsedHeight + expandedContent.height + marginOffsetY
    
    /**
     * Loader to hold content shown when collapsed (and also expanded)
     */
    property alias collapsedLoader: collapsedContent
    
    /**
     * Loader to hold content shown only when expanding
     */
    property alias extensionLoader: expandedContent

    /**
     * Boolean to determine if the stimulus area should be shown.
     */
    property bool isClickable: true
    
    /**
     * 
     */
    signal clicked();
    
    onExpandedChanged: { state = expanded ? "expanded" : "collapsed" }
    
    /**
      * Main level box
      */
    Rectangle {
        id: collapsed_box

        width:  collapsedContent.width
        height: collapsedContent.height
        anchors.top: root.top
        anchors.left: root.left
        radius: main.defaultRadius

        color: expanded ? main.colorHighlightBackground : main.colorButtonBackground
        border.color: main.colorUIBorder
        border.width: 2
        z: parent.z-2
    }

    /**
      * Expanded Box under main box
      */
    Rectangle {
        id: expanded_box

        width:  expandedContent.width
        height: expandedContent.height
        radius: main.defaultRadius

        anchors.top: collapsed_box.bottom
        anchors.topMargin: root.marginOffsetY
        anchors.left: collapsed_box.left
        anchors.leftMargin: root.marginOffsetX

        color: main.colorButtonBackground
        border.color: main.colorUIBorder
        border.width: 2

        visible: !collapsed_box.expanded
        z:collapsed_box.z-1
    }

    /**
     * Simply have two states: collapsed and expanded
     * Use the appropriate visibility for each state
     */
    states: [
        State {
            name: "collapsed"
            PropertyChanges {
                target: root
                height: collapsedHeight
                width: collapsedWidth
            }
            PropertyChanges {
                target: expanded_box
                visible: false
            }
        },
        State {
            name: "expanded"
            PropertyChanges {
                target: root
                height: expandedHeight
                width: expandedWidth
            }
            PropertyChanges {
                target: expanded_box
                visible: true
            }
        }
    ]
    /**
     * Animations when changing states
     */
    /*
    transitions: [
        Transition {
            ParallelAnimation {
                PropertyAnimation {
                    target: root
                    property: "height"
                    duration: expandTime
                }
            }
        }
    ]
    */
    
    /**
     * Returns true when state is expanded, otherwise false
     */
    function isExpanded() {
        return expanded;
    }
    /**
     * Sets state to collapse the box
     */
    function collapse() {
        expanded = false;
    }
    /**
     * Sets state to expand the box
     */
    function expand() {
        expanded = true;
    }
    
    MouseArea {
        anchors.fill: parent
        enabled: clickToExpand && isClickable
        onClicked: {
            expanded = !expanded;
            root.clicked();
            buzzer.buttonPress()
        }
        Rectangle {
            anchors.fill: parent
            color: main.colorGenericStimulusArea
            opacity: 0.3
            visible: main.showStimulusArea && isClickable
        }
    }
    
    Column {
        anchors.fill: parent
        clip: true
        
        Loader {
            id: collapsedContent
            asynchronous: true
            onSourceComponentChanged: {
                height = sourceComponent.height
                width = sourceComponent.width
            }
        }
        
        Loader {
            id: expandedContent
            asynchronous: true
            clip: true
            anchors.left: parent.left
            anchors.leftMargin: root.marginOffsetX
            onSourceComponentChanged: {
                height = sourceComponent.height
                width = sourceComponent.width
            }
        }
    }
}
