import QtQuick 2.12
import "../qml"

/* Data Table Element
* Show a series of ListModels as columns of data in a Table
*/

Item {
    id: root

    /* list of ListModels, one for each column of the table
    // properties may be attached to each model (as shown in demoModel below), to control the following:
    //      heading - text for header
    // The "label" role of each ListElement is used as the label data in each cell in the column.
    // The "value" role of each ListElement is used as the value data in each cell in the column.
    // The "isDisplayed" role of each ListElement is used to display or hide each cell in the column.
    // Each model in the list is another column in the table.
    */

    // REQUIRED property: for translation to work (via qsTranslate), it must know where the external model is stored.
    // Without this you will likely see blank strings in your data table.
    property string sourceFile: ""

    property alias dataDisplayModel: selectDataModel
    property bool selectable: false

    property string ruleSourceName: "Components/dotted_line_separator_single"

    // default column width, used if column model has no defined width// Width of each separator
    property real separatorWidth: 3 * main.hScaleFactor
    property int colWidth: width / dataDisplayModel.headings.length  - separatorWidth
    property int headerHeight: height * 0.07
    property int hAlign: Text.AlignHCenter

    property string distanceLabel
    property string paceLabel
    property string avgPaceLabel
    property string unitsLabel
    property string distanceClimbedLabel
    property string paceLabelTranslated
    property string unitsLabelTranslated

    property bool wasHRAutoSelected: false
    property alias dataIdxEnum: displayIndexEnum

    // re-translate the strings when the language changes
    Connections {
        target: application
        onLanguageChanged: {
            root.unitsLabelTranslated = qsTr(root.unitsLabel) + application.emptyString
            updateLabels()
        }
    }

    Connections {
        target: workoutScreen
        onUnitStateChanged: {
            updateLabels()
        }
    }

    Connections {
        target: heartRateManager
        onHeartRateAvailableChanged: {
            if(heartRateManager.heartRateAvailable) {
                if(root.dataDisplayModel.get(dataIdxEnum.bpm).isSelected === false) {
                    if(root.selectedCount === 6) {
                        // Drop last entry to make room for heart rate
                        root.removeLastEntry()
                    }
                    root.toggleSelection(dataIdxEnum.bpm,!heartRateManager.heartRateAvailable) // Turn on Heart Rate
                    root.wasHRAutoSelected = true
                }
            } else if(root.dataDisplayModel.get(dataIdxEnum.bpm).isSelected === true && root.wasHRAutoSelected ) {
                    root.toggleSelection(dataIdxEnum.bpm,!heartRateManager.heartRateAvailable) // Turn off Heart Rate
            }
        }
    }

    property int selectedCount: 0

    onSelectedCountChanged: {
        if(selectedCount === 0) {
            resetDefaultSelection()
        }
    }

    function reset() {
        root.wasHRAutoSelected = false
        resetDefaultSelection()
        updateLabels()
    }

    function toggleSelection(index, currentIsSelected) {
        if(selectedCount === 0 && currentIsSelected === true) {
            return;
        }

        if(selectedCount === 6 && currentIsSelected === false) {
            return;
        }
        root.dataDisplayModel.setProperty(index, "isSelected", !currentIsSelected)

        if(root.dataDisplayModel.get(index).isSelected === true) {
            selectedCount++;
        } else {
            selectedCount--;
        }
    }

    function resetDefaultSelection() {
        if (!selectable)
            return

        var i;

        // Reset all selection toggles
        for(i = 0; i < root.dataDisplayModel.count; i++) {
            root.dataDisplayModel.setProperty(i, "isSelected", false);
        }

        // Set default selection toggles
        root.dataDisplayModel.setProperty(dataIdxEnum.timeElapsed, "isSelected", true); // Time Elapsed

        if(gem.appleWatchConnected && gem.connected === 1) {
            root.dataDisplayModel.setProperty(dataIdxEnum.activeCalories, "isSelected", true); // Active Calories
            root.dataDisplayModel.setProperty(dataIdxEnum.totalCalories, "isSelected", true); // Total Calories
            selectedCount = 4;
        } else {
            root.dataDisplayModel.setProperty(dataIdxEnum.calories, "isSelected", true); // Calories
            selectedCount = 3;
        }

        if(root.dataDisplayModel.get(dataIdxEnum.distance).isDisplayed)
            root.dataDisplayModel.setProperty(dataIdxEnum.distance, "isSelected", true); // Distance
        else if(root.dataDisplayModel.get(dataIdxEnum.distanceRemaining).isDisplayed)
            root.dataDisplayModel.setProperty(dataIdxEnum.distanceRemaining, "isSelected", true); // Distance Remaining
        else if(root.dataDisplayModel.get(dataIdxEnum.distanceClimbed).isDisplayed)
            root.dataDisplayModel.setProperty(dataIdxEnum.distanceClimbed, "isSelected", true); // Distance Climbed
        else // No distances displayed
            selectedCount--;

        if (heartRateManager.heartRateAvailable) {
            root.dataDisplayModel.setProperty(dataIdxEnum.bpm, "isSelected", true); // Heart Rate
            selectedCount++
        }

        if(workoutProgram.isHRCProgram) {
            root.dataDisplayModel.setProperty(dataIdxEnum.targetBPM, "isSelected", true); // Target Heart Rate
            selectedCount++
        }

        if(workoutProgram.isFloorsWorkout){


        }
    }

    function removeLastEntry() {
        var c, i;
        var item;

        for(i = root.dataDisplayModel.count-1; i >=0 ; i--) {
            item = root.dataDisplayModel.get(i);

            if(item.isSelected === true && item.columnIndex !== 4 ) {  // Don't remove HR related items
                root.dataDisplayModel.setProperty(i, "isSelected", false)
                selectedCount--
                return
            }
        }
    }

    function updateLabels() {
        root.distanceLabel = workoutScreen.unitState === "ENGLISH" ? qsTr("MILES") : qsTr("KM")
        root.paceLabel = workoutScreen.unitState === "ENGLISH" ? qsTr("PACE/MILE") : qsTr("PACE/KM")
        root.avgPaceLabel = workoutScreen.unitState === "ENGLISH" ? qsTr("AVG PACE/MILE") : qsTr("AVG PACE/KM")
        root.distanceClimbedLabel = workoutScreen.unitState === "ENGLISH" ? qsTr("FEET CLIMBED") : qsTr("METERS CLIMBED")

        root.unitsLabel = main.isStepmill ? qsTr("SPM") : workoutScreen.unitState === "ENGLISH" ? qsTr("MPH") : qsTr("KPH")
        root.unitsLabelTranslated = qsTr(root.unitsLabel) + application.emptyString

        selectDataModel.setProperty(displayIndexEnum.distance, "label", root.distanceLabel)
        selectDataModel.setProperty(displayIndexEnum.pace, "label", root.paceLabel)
        selectDataModel.setProperty(displayIndexEnum.distanceClimbed, "label", root.distanceClimbedLabel)
    }

    Component.onCompleted: reset();

    QtObject{
        id: displayIndexEnum
        property int distance : 0
        property int distanceRemaining : 1
        property int distanceClimbed : 2
        property int avgIncline : 3
        property int strideCount : 4
        property int stepsClimbed : 5
        property int floorsClimbed: 6
        property int floorsRemaining: 7
        property int timeElapsed : 8
        property int timeRemaining : 9
        property int segmentTimeRemaining : 10
        property int pace : 11
        property int avgPace : 12
        property int avgSpeed : 13
        property int maxSpeed : 14
        property int calories : 15
        property int calPerHr : 16
        property int activeCalories : 17
        property int totalCalories : 18
        property int mets : 19
        property int watts : 20
        property int avgWatts : 21
        property int bpm : 22
        property int maxBPM : 23
        property int avgBPM : 24
        property int targetBPM : 25
    }

    ListModel {
        id: selectDataModel //dataDisplayModel alias
        property var headings: [qsTr("DISTANCE") + application.emptyString, qsTr("TIME") + application.emptyString,
                                qsTr("CALORIES") + application.emptyString, qsTr("METS") + application.emptyString,
                                qsTr("HEART RATE") + application.emptyString]

        ListElement{ label: qsTr("DISTANCE"); isSelected: false; isDisplayed: true; value: "0"; isHR: false; columnIndex: 0}
        ListElement{ label: qsTr("DISTANCE REMAINING"); isSelected: false; isDisplayed: true; value: "0"; isHR: false; columnIndex: 0}
        ListElement{ label: qsTr("DISTANCE CLIMBED"); isSelected: false; isDisplayed: true; value: "0"; isHR: false; columnIndex: 0}
        ListElement{ label: qsTr("AVG INCLINE"); isSelected: false; isDisplayed: true; value: "0"; isHR: false; columnIndex: 0}
        ListElement{ label: qsTr("STRIDE COUNT"); isSelected: false; isDisplayed: true; value: "0"; isHR: false; columnIndex: 0}
        ListElement{ label: qsTr("STEPS"); isSelected: false; isDisplayed: true; value: "0"; isHR: false; columnIndex: 0}
        ListElement{ label: qsTr("FLOORS"); isSelected: false; isDisplayed: true; value: "0"; isHR: false; columnIndex: 0}
        ListElement{ label: qsTr("FLOORS REMAINING"); isSelected: false; isDisplayed: false; value: "0"; isHR: false; columnIndex: 0}

        ListElement{ label: qsTr("TIME ELAPSED"); isSelected: false; isDisplayed: true; value: "0"; isHR: false; columnIndex: 1}
        ListElement{ label: qsTr("TIME REMAINING"); isSelected: false; isDisplayed: true; value: "0"; isHR: false; columnIndex: 1}
        ListElement{ label: qsTr("SEGMENT TIME"); isSelected: false; isDisplayed: true; value: "0"; isHR: false; columnIndex: 1}
        ListElement{ label: qsTr("PACE"); isSelected: false; isDisplayed: true; value: "0"; isHR: false; columnIndex: 1}
        ListElement{ label: qsTr("AVG PACE"); isSelected: false; isDisplayed: true; value: "0"; isHR: false; columnIndex: 1}
        ListElement{ label: qsTr("AVG SPEED"); isSelected: false; isDisplayed: true; value: "0"; isHR: false; columnIndex: 1}
        ListElement{ label: qsTr("MAX SPEED"); isSelected: false; isDisplayed: true; value: "0"; columnIndex: 1}

        ListElement{ label: qsTr("CALORIES"); isSelected: false; isDisplayed: true; value: "0"; isHR: false; columnIndex: 2}
        ListElement{ label: qsTr("CALORIES/HR"); isSelected: false; isDisplayed: true; value: "0"; isHR: false; columnIndex: 2}
        ListElement{ label: qsTr("ACTIVE CALORIES"); isSelected: false; isDisplayed: true; value: "0"; isHR: false; columnIndex: 2}
        ListElement{ label: qsTr("TOTAL CALORIES"); isSelected: false; isDisplayed: true; value: "0"; isHR: false; columnIndex: 2}

        ListElement{ label: qsTr("METS"); isSelected: false; isDisplayed: true; value: "0"; isHR: false; columnIndex: 3}
        ListElement{ label: qsTr("WATTS"); isSelected: false; isDisplayed: true; value: "0"; isHR: false; columnIndex: 3}
        ListElement{ label: qsTr("AVG WATTS"); isSelected: false; isDisplayed: true; value: "0"; isHR: false; columnIndex: 3}

        ListElement{ label: qsTr("BPM"); isSelected: false; isDisplayed: true; value: "0"; isHR: true; columnIndex: 4}
        ListElement{ label: qsTr("MAX BPM"); isSelected: false; isDisplayed: true; value: "0"; isHR: false; columnIndex: 4}
        ListElement{ label: qsTr("AVG BPM"); isSelected: false; isDisplayed: true; value: "0"; isHR: false; columnIndex: 4}
        ListElement{ label: qsTr("TARGET BPM"); isSelected: false; isDisplayed: true; value: "0"; isHR: false; columnIndex: 4}
    }

    Component {
        id: entryDelegate

        Item {
            property int colIndex: ListView.view.colIndex

            height: visible ? ListView.view.height*0.2 : 0
            width: ListView.view.width
            visible: model.isDisplayed && model.columnIndex === colIndex

            // These are the dimensions of the content that you can actually see
            property real visibleWidth: width*0.9
            property real visibleHeight: height*0.8

            Rectangle {
                width: parent.visibleWidth
                height: parent.visibleHeight
                radius: main.defaultRadius
                anchors.horizontalCenter: parent.horizontalCenter
                anchors.verticalCenter: parent.verticalCenter
                color: isSelected ? main.colorSelected : "transparent"
                opacity: 1
            }

            NumberLabel {
                id: value
                height: parent.visibleHeight/2
                width: parent.visibleWidth
                anchors.horizontalCenter: parent.horizontalCenter
                anchors.verticalCenter: parent.verticalCenter
                anchors.verticalCenterOffset: -height / 2
                visible: true
                font.pixelSize: height * 0.8
                horizontalAlignment: Text.AlignHCenter
                verticalAlignment: Text.AlignVCenter
                text: model.value
            }

            TextLabel {
                id: label
                height: parent.visibleHeight/2
                width: 0.95*parent.visibleWidth
                anchors.horizontalCenter: parent.horizontalCenter
                anchors.verticalCenter: parent.verticalCenter
                anchors.verticalCenterOffset: height / 2
                font.pixelSize: height * 0.4
                horizontalAlignment: root.hAlign
                verticalAlignment: Text.AlignVCenter
                text: {
                    if (model.label === "AVG PACE")
                        return main.isTreadmill ? root.avgPaceLabel : qsTranslate(root.sourceFile, "AVG PACE (RPM)") + application.emptyString
                    else if (model.label === "AVG SPEED")
                        return qsTranslate(root.sourceFile, "AVG SPEED") + "(" + unitsLabelTranslated + ")" + application.emptyString
                    else if (model.label === "MAX SPEED")
                        return ((main.isTreadmill || main.isStepmill) ? qsTranslate(root.sourceFile, "MAX SPEED") + "(" + unitsLabelTranslated + ")" : qsTranslate(root.sourceFile, "MAX PACE (RPM)") ) + application.emptyString
                    else
                        return qsTranslate(root.sourceFile, model.label) + application.emptyString
                }
                wrapMode: Text.WordWrap
            }

            MouseArea {
                id: mouseArea
                visible: root.selectable
                enabled: root.selectable
                width: parent.visibleWidth
                height: parent.visibleHeight
                anchors.horizontalCenter: parent.horizontalCenter
                anchors.verticalCenter: parent.verticalCenter
                onClicked: {
                    toggleSelection(model.index, model.isSelected)
                    root.wasHRAutoSelected = false
                    buzzer.buttonPress()
                }

                Rectangle {
                    anchors.fill: parent
                    color: main.colorGenericStimulusArea
                    opacity: 0.3
                    visible: main.showStimulusArea
                }
            }
        }

    }

    Component {
        id: colview

        Item {
            id: columnItem

            property variant model

            width: root.colWidth
            height: root.height

            // column header text
            TextLabel {
                id: headingText
                text: qsTranslate(root.sourceFile, columnItem.model.headings[index])
                font.pixelSize: height*0.4
                width: parent.width
                height: root.headerHeight
                horizontalAlignment: root.hAlign
                verticalAlignment: Text.AlignVCenter
                anchors{ top: parent.top; topMargin: 0; left: parent.left ; leftMargin: 0;}
            }

            ScalableImage {
                id: verticalRule
                width: root.separatorWidth
                height: 0.8 * (parent.height - root.headerHeight)
                anchors.top: headingText.bottom
                anchors.topMargin: 5 * main.vScaleFactor
                anchors.right: parent.right
                anchors.rightMargin: -width/2

                fillMode: Image.TileVertically
                horizontalAlignment: Image.AlignHCenter
                verticalAlignment: Image.AlignTop

                sourceName: ruleSourceName
                visible: index < columnItem.model.headings.length - 1
            }

            Rectangle {
                id: horizontalRule
                width: parent.width
                height: 2 * main.vScaleFactor
                color: main.colorUIBorder
                anchors { top: headingText.bottom ; left: parent.left }
            }

            ListView {
                id: colListView
                property int colIndex: index
                interactive: selectable
                clip: true

                width: parent.width
                height: parent.height - root.headerHeight
                delegate: entryDelegate
                anchors{ top: headingText.bottom; topMargin: 0; left: parent.left ; leftMargin: 0 }
                model: root.dataDisplayModel
            }
        }
    }

    // row of table columns
    Row {
        spacing: 0
        Repeater {
            model: root.dataDisplayModel.headings.length
            delegate: colview

            onItemAdded: { // assign a model from the array to each column
                item.model = root.dataDisplayModel
            }
        }
    }
}

