import QtQuick 2.12

// This element manages audio priority and tracks requests from audio sources to play audio to user.

// The AudioManager accepts requests to play audio via several xxxRequest functions, where xxx is ipod, tv, va, etc.
// It responds by modifying a set of xxxAllowed properties, which indicate that it is OK for the corresponding source to play audio.
// The ipod source is coded as the highest priority source, and if playing, all other sources are denied.

Item {
    id: audioManager

    property int volume: volumeDefault // Volume percent
    property int volumeDefault: 100 * vol_setting / (vol_max - vol_min)  // default volume
    property int vol_setting: 5 // default volume setting
    property int vol_max: 20
    property int vol_min: 0
    property bool mute: false
    property bool btconnected: false
    property bool ipodPlaying: false
    property bool ipodPlayingVideo: false
    property bool tvPlaying: false
    property bool virtActivePlaying: false
    property bool browserPlaying: false

    property bool ipodAllowed: false
    property bool tvAllowed: false
    property bool virtActiveAllowed: false
    property bool ipodVideoAllowed: false
    property bool tvVideoAllowed: false
    property bool virtActiveVideoAllowed: false
    property bool browserAllowed: false
    property bool btAllowed: false
    property string currentMuxSource: ""

    property variant audioSources: []
    property string videoSource: ""

    property bool audioPlaying: audioSources.length > 0
    property bool debug: false

    function debugPrint(message) {
        if(debug)
            errorManager.debugMessage("AudioManager: " + message)
    }

    function setMuteByView(view) {
        switch(view) {
        case "TV_VIEW":
            tvAllowed = true
            if(bluetoothDeviceAudioWatcher.available === true) {
                btMute.setMute(1)
            }

            if (browserAudioWatcher.soundIsPlaying) {
                browserMute.setMute(1)
            }
            break;
        case "WEB_VIEW":
            if (browserAudioWatcher.soundIsPlaying) {
                if(bluetoothDeviceAudioWatcher.available === true) {
                    btMute.setMute(1)
                }
                browserMute.setMute(0)
            } else {
                browserMute.setMute(1)
                if(bluetoothDeviceAudioWatcher.available === true) {
                    btMute.setMute(0)
                }
            }
            break;
        default:
            if(bluetoothDeviceAudioWatcher.available === true) {
                btMute.setMute(0)
            }

            if (browserAudioWatcher.soundIsPlaying) {
                browserMute.setMute(1)
            }

            break;
        }
    }

    Connections {
        target: browserAudioWatcher
        onSoundIsPlayingChanged: {
            setMuteByView(workoutScreen.container.currentViewName)
        }
    }

    Connections {
        target: bluetoothDeviceAudioWatcher
        onAvailableChanged: {
            setMuteByView(workoutScreen.container.currentViewName)
        }
    }

    Connections {
        target: workoutScreen.container
        onCurrentViewNameChanged: {
            setMuteByView(workoutScreen.container.currentViewName)
        }
    }

    Connections {
        target: ipod_window
        onVisibleChanged: {
            if(ipod_window.visible === false) {
                setMuteByView(workoutScreen.container.currentViewName)
            } else {
                if(workoutScreen.container.currentViewName === "TV_VIEW") {
                    tvAllowed = false
                    if(bluetoothDeviceAudioWatcher.available === true) {
                        btMute.setMute(0)
                    }

                }
            }
        }
    }

    function reset() {
        debugPrint("reset")

        ipodAllowed = false
        virtActiveAllowed = false
        tvAllowed = false

        virtActiveVideoAllowed = false
        ipodVideoAllowed = false
        tvVideoAllowed = false
        browserAllowed = false

        btAllowed = false

        setMux("none")
        audioManager.volume = audioManager.volumeDefault

        rebuildSourceList()
    }

    function setVolume(percent) {
        audioManager.volume = percent
    }

    function setMute(muted) {
        audioManager.mute = muted
    }

    onMuteChanged: {
    }

    onVolumeChanged: {
        audioManager.mute = false
    }

    function setMux(source) {
        debugPrint("set audio mux: " + source)
        switch(source) {
            case "none":{
                if (currentMuxSource !== "none") {
                    currentMuxSource = "none"
                }
                break;
            }
            case "ipod":{
                if (currentMuxSource !== "ipod") {
                    currentMuxSource = "ipod"
                }
                break;
            }
        }
    }

    function resyncMuxSource() {
        if (ipodAllowed || ipodVideoAllowed)
            setMux("ipod")
        else
            setMux("none")
    }

    function getCurrentSource() {
            if (audioManager.ipodAllowed || audioManager.ipodVideoAllowed)
                return "ipod"
            else if (audioManager.btAllowed)
                return "bt"
            else if (audioManager.tvAllowed)
                return "tv"
            else if (audioManager.browserAllowed)
                return "browser"
            else
                return ""
    }

    function nextSource() {
        var currentSource = getCurrentSource()
        // find source in list
        var currentSourceIndex = -1
        for (var i = 0; i < audioSources.length; i++) {
            if (audioSources[i] === currentSource) {
                currentSourceIndex = i
                break
            }
        }

        var nextSrc = audioSources[(currentSourceIndex + 1) % audioSources.length]

        if (nextSrc)
            enableSource(nextSrc)
    }

    function ipodRequest() {
        debugPrint("ipod request")
        if(!ipodAllowed)
            enableSource("ipod")
    }

    function ipodVideoRequest() {
        debugPrint("ipod video request")
        if(!ipodVideoAllowed) {
            enableSource("ipod")
            tvVideoAllowed = false
            virtActiveVideoAllowed = false
            ipodVideoAllowed = true
            browserAllowed = false
            videoSource = "ipod"
        }
    }

    function tvRequest() {
        debugPrint("tv request")
        _lastRequested = "tv"
        if(!tvAllowed)
            enableSource("tv")
    }

    function vaRequest() {
        debugPrint("va request")
        _lastRequested = "va"
        if(!virtActiveAllowed)
            enableSource("va")
    }

    function browserRequest() {
        _lastRequested = "browser"
        if(!browserAllowed)
            enableSource("browser")
    }

    function btRequest() {
        if(!btAllowed)
            enableSource("bt")
    }

    Component.onCompleted: reset()

    property string _lastRequested   // string name of last source which will be re-enabled when ipod stops

    onTvPlayingChanged: rebuildSourceList()
    onVirtActivePlayingChanged: rebuildSourceList()
    onIpodPlayingChanged: rebuildSourceList()
    onBtconnectedChanged: rebuildSourceList()
    onBrowserPlayingChanged: { console.log("AudioManager browserPlaying", browserPlaying); rebuildSourceList() }

    onIpodAllowedChanged: resyncMuxSource()
    onIpodVideoAllowedChanged: resyncMuxSource()

    onVideoSourceChanged: debugPrint("Video Source:" + videoSource)

    function enableSource(name) {
        debugPrint("enable audio source " + name)

        switch(name) {
            case "ipod": {
                tvAllowed = false
                virtActiveAllowed = false
                ipodAllowed = true
                browserAllowed = false
                btAllowed = false

                break;
            }
            case "tv": {
            
                ipodAllowed = false
                virtActiveAllowed = false
                tvAllowed = true
                virtActiveVideoAllowed = false
                ipodVideoAllowed = false
                tvVideoAllowed = true
                browserAllowed = false
                btAllowed = false

                videoSource = "tv"

                setMux("none")

                break;
            }
            case "va": {
                ipodAllowed = false
                tvAllowed = false
                virtActiveAllowed = true
                ipodVideoAllowed = false
                tvVideoAllowed = false
                virtActiveVideoAllowed = true
                browserAllowed = false
                btAllowed = false

                videoSource = "va"

                setMux("none")

                break;
            }
            case "browser": {
                ipodAllowed = false
                tvAllowed = false
                virtActiveAllowed = false
                ipodVideoAllowed = false
                //tvVideoAllowed = false
                //virtActiveVideoAllowed = false
                browserAllowed = true
                btAllowed = false

                setMux("none")

                break;
            }

            case "bt": {
                ipodAllowed = false
                tvAllowed = false
                virtActiveAllowed = false
                browserAllowed = false
                btAllowed = true

                setMux("none")
                break
            }
        }
    }

    // update string-based list of currently playing sources
    function rebuildSourceList() {
        var newSources = ""
        newSources = []
        if(ipodPlaying)
            newSources = newSources.concat(["ipod"])
        if(btconnected)
            newSources = newSources.concat(["bt"])
        if(tvPlaying) {
            newSources = newSources.concat(["tv"])
        } else if (videoSource === "tv") {
            videoSource = ""
        }

        if (browserPlaying)
            newSources = newSources.concat(["browser"])

        audioSources = newSources
    }
}
