import QtQuick 2.12
import ".."
import "DiagnosticsMenu.js" as DiagnosticsMenu
import Components 1.0

Item {
    id: diagnosticsMain
    width: main.screenWidth
    height: main.screenHeight
    property int accessLevel: -1
    property int diagnosticAccessCode: 0

    onDiagnosticAccessCodeChanged: {
        setAccessLevel()
    }

    QtObject {
        id: accessCodes
        // Access codes to restrict access to diagnostics options and settings
        readonly property int manager: 10101
        readonly property int maintenance: 20173
        readonly property int all: 38462
        property string accessTitle: qsTr("No Access") + application.emptyString
    }

    function setAccessLevel() {
        switch (diagnosticsMain.diagnosticAccessCode) {
        case accessCodes.manager:
            diagnosticsMain.accessLevel = 0;
            accessCodes.accessTitle = qsTr("MANAGER") + application.emptyString
            break;
        case accessCodes.maintenance:
            diagnosticsMain.accessLevel = 1;
            accessCodes.accessTitle = qsTr("TECHNICIAN") + application.emptyString
            break;
        case accessCodes.all:
            diagnosticsMain.accessLevel = 2;
            accessCodes.accessTitle = qsTr("ALL ACCESS") + application.emptyString
            break;
        default:
            diagnosticsMain.accessLevel = -1;
            if(diagnosticsMain.diagnosticAccessCode > 0)
                console.log("DiagnosticsMain: Incorrect Access Code entered " + diagnosticsMain.diagnosticAccessCode)
        }
        menureset();
    }

    function setTopMenu() {
        menu.clear()
        var menudef
        if (machineinfo.productTypeString == "Treadmill") {
            menudef = DiagnosticsMenu.diagnosticsMenuMainTread(diagnosticsMain.accessLevel)
        } else if (machineinfo.productTypeString.search("Elliptical") != -1) {
            if (main.isCSA) {
                menudef = DiagnosticsMenu.diagnosticsMenuMainStride(diagnosticsMain.accessLevel)
            } else {
                menudef = DiagnosticsMenu.diagnosticsMenuMain(diagnosticsMain.accessLevel)
            }
        } else {
            menudef = DiagnosticsMenu.diagnosticsMenuMain(diagnosticsMain.accessLevel)
        }

        var menuitem
        for (var i=0; i < menudef.children.length ; ++i ) {
            menuitem = menudef.children[i]
            var newitem = {
                "label" : application.translateText("Diagnostics", menuitem.label),
                "qmlSource": menuitem.qmlSource,
                "children": menuitem.children
            }
            menu.append(newitem)
        }
    }

    property string headingText: accessCodes.accessTitle

    property alias menuIndex: sideMenuListView.currentIndex

    property int currentSelectedIndex: -1

    signal close()

    function exitButtonPressedHandler() {
        diagnosticsMain.diagnosticAccessCode = 0
        close()
    }

    function menureset(){
        setTopMenu()
        menuIndex = 0
        diagnosticsMain.currentSelectedIndex = 0
    }

    function updateStrings () {
        setTopMenu()
    }

    // re-translate the menu strings when the language changes
    Connections {
        target: application
        onLanguageChanged: { updateStrings() }
    }

    Component.onCompleted: {  // set the top level menu at startup
        menureset()
    }

    onMenuIndexChanged: {
        secondaryMenu.menudef = menu.get(diagnosticsMain.menuIndex)
        secondaryMenu.contentSource = menu.get(diagnosticsMain.menuIndex).qmlSource
    }

    ListModel {
        id: menu
    }

    Component{
        id: menuItemDelegate

        StandardButton {
            selected: diagnosticsMain.menuIndex == index
            buttonText: model.label !== undefined ? qsTranslate("Diagnostics", model.label) + application.emptyString : ""
            label.font.pixelSize: 16 * main.vScaleFactor
            width: 0.75 * parent.width
            height: 79 * main.vScaleFactor
            anchors.horizontalCenter: parent.horizontalCenter
            onClicked: {
                if(!selected) {
                    buzzer.buttonPress()
                    diagnosticsMain.menuIndex = index
                }
            }
        }
    }

    // side Menu
    Item{
        id:sideMenuSection
        anchors.left: diagnosticsMain.left
        anchors.top: diagnosticsMain.top
        anchors.bottom: diagnosticsMain.bottom
        width: 0.2 * diagnosticsMain.width

        ScalableImage {
            id: menuBackground
            anchors.fill: parent
            fillMode: Image.PreserveAspectCrop
            sourceName: "StandardPage/View_Background"
            z: parent.z - 1
        }

        TextLabel{
            id: sideMenuSectionHeading

            anchors.left: parent.left
            anchors.right: parent.right
            anchors.top: parent.top
            anchors.topMargin: 10 * main.vScaleFactor
            height: parent.height * 0.07
            text: qsTranslate("Diagnostics", headingText) + application.emptyString

//            scale: paintedWidth > width ? (width / paintedWidth) : 1
//            transformOrigin: Item.Left
            horizontalAlignment: Text.AlignHCenter
            verticalAlignment: Text.AlignVCenter
            font.pixelSize: 24 * main.vScaleFactor
        }

        ListView{
            id: sideMenuListView
            anchors{
                top: sideMenuSectionHeading.bottom
                topMargin: 10 * main.vScaleFactor
                bottom: exitButton.top
                bottomMargin: 30 * main.vScaleFactor
                left: parent.left
                right: parent.right
            }

            spacing: 20 * main.vScaleFactor
            model: menu
            delegate: menuItemDelegate

            interactive: false
        }

        StandardButton {
            id: exitButton
            width: 0.75 * parent.width
            height: 70 * main.vScaleFactor
            anchors.horizontalCenter: parent.horizontalCenter
            anchors.bottom: parent.bottom
            anchors.bottomMargin: 20 * main.vScaleFactor
            buttonText: qsTr("Exit") + application.emptyString
            onClicked: diagnosticsMain.exitButtonPressedHandler()
        }
    }

    DiagnosticsSecondary {
        id: secondaryMenu

        topMargin: sideMenuSectionHeading.height + 10 * main.vScaleFactor
        anchors {
            left: sideMenuSection.right
            leftMargin: -1
            right: parent.right
            top: parent.top
            bottom: parent.bottom
        }

    }
}
