// Diagnostics Software Update Page

import QtQuick 2.12
import StorageMgr 1.0
import Process 0.1
import ".."
import Components 1.0

Item {
    id: root
    anchors.fill: parent

    property int fontSize: 20 * main.vScaleFactor
    property int itemSpacing: 5 * main.vScaleFactor
    property string firmwareVersion: nucleusfirmwareloader.firmwareVersionString
    property string firmwareUpdateFile: nucleusfirmwareloader.updateFile
    property bool firmwareUpdating: nucleusfirmwareloader.isUpdating
    property bool firmwareUpdateAvailable: nucleusfirmwareloader.isUpdateAvailable

    property string cursorState: "firmware" // check, firmware, or software

    onFirmwareUpdatingChanged: {
        if(firmwareUpdating)
            firmwareUpdatingDialog.show()
        else
            firmwareUpdatingDialog.hide()
    }

    Connections {
        target: nucleusfirmwareloader
        onFault: {
            print("Diagnostics Update: " + message)
            firmwareUpdatingDialog.hide()
            bootloadErrorReceivedDialogText.text = message
            bootloadErrorReceivedDialog.show()
        }
        onStatus: {
            firmwareUpdatingDialog.text = message
        }
        onUpdateSuccessful: {
            main.restartRequired = true
            restartMsg.visible = true
        }
    }

    signal close()

    ModalDialog{
        id:bootloadErrorReceivedDialog
        blockKeys: true
        dismissable: true

        Text {
            id: bootloadErrorReceivedDialogText
            anchors.fill: parent
            anchors.verticalCenter: parent.verticalCenter
            color: main.colorFontLight
            anchors.horizontalCenter: parent.horizontalCenter
            horizontalAlignment: Text.AlignHCenter
            verticalAlignment: Text.AlignVCenter
            wrapMode: Text.WordWrap
            font.bold: false
            font.pixelSize: 50 * main.vScaleFactor
            z: bootloadErrorReceivedDialog.z + 1

        }
        MouseArea{
            anchors.fill: parent
            onClicked: {
                bootloadErrorReceivedDialog.hide()
                buzzer.buttonPress()
            }
            Rectangle {
                anchors.fill: parent
                color: main.colorGenericStimulusArea
                opacity: 0.3
                visible: main.showStimulusArea
            }
        }

        Component.onCompleted: hide()
    }

    Item{
        id: informationSection
        width: parent.width
        height: childrenRect.height
        anchors.horizontalCenter: parent.horizontalCenter
        anchors.top: root.top
        anchors.topMargin: 50 * main.vScaleFactor
        Row{
            spacing: 50 * main.hScaleFactor

            Text{
                text: qsTr("Nucleus Firmware Version") + application.emptyString
                font.pointSize: fontSize
            }

            Text{
                text: machineinfo.nucleusFirmwareVersion === "" ? qsTr("NOT AVAILABLE") + application.emptyString : machineinfo.nucleusFirmwareVersion
                font.pointSize: fontSize
            }
        }
    }

    Item{
        id: firmwareUpdateSection

        width: parent.width
        height: childrenRect.height


        anchors {
            top: informationSection.bottom ; topMargin: 50 * main.vScaleFactor
            left: parent.left ; leftMargin: 80 * main.hScaleFactor
        }

        Text{
            id: hexFileStatusText
            font.pointSize: fontSize
            anchors {
                left: bootloadButton.right ;
                leftMargin: 15 * main.hScaleFactor;
                verticalCenter: bootloadButton.verticalCenter
            }
            visible: !restartMsg.visible
            text:
            {
                if( bootloadButton.visible ) {
                    return qsTr("Firmware to load: ") + application.emptyString + firmwareVersion + "\n" + firmwareUpdateFile
                }
                else
                    return ""
            }
        }

        StandardButton {
            id: bootloadButton

            buttonText: qsTr("Load Firmware") + application.emptyString
            visible: firmwareUpdateAvailable && !restartMsg.visible
            onVisibleChanged: {
                if (visible)
                    root.cursorState = "firmware"
            }

            width: 150 * main.hScaleFactor
            height: 78 * main.vScaleFactor
            anchors.left: parent.left
            anchors.top: restartMsg.bottom

            onClicked: {
                print("Diagnostics Update: Loading firmware:" + firmwareUpdateFile)
                nucleusfirmwareloader.startFirmwareLoad(firmwareUpdateFile)
            }
        }

        Text {
            id: restartMsg
            visible: false
            text: qsTr("Firmware updated. Restart console.") + application.emptyString
            font.pointSize: fontSize
            anchors { horizontalCenter: parent.horizontalCenter ; top: parent.top }
        }
    }

    Text {
        id: instructions

        width: (parent.width - 100) * main.hScaleFactor

        anchors { bottom: parent.bottom ; bottomMargin: 90 * main.vScaleFactor ;  horizontalCenter: parent.horizontalCenter }
        horizontalAlignment: Text.AlignLeft
        wrapMode: Text.WordWrap
        visible: !restartMsg.visible
        text: {
            if(root.firmwareUpdating)
                return qsTr("Firmware is now being updated.  DO NOT power off the machine while software update is taking place.")
            else
                return ""
        }
    }

    ModalDialog{
        id: waitingForBootloaderDialog
        blockKeys: true

        //visible: bootloader.connected && bootloader.progress > 0

        Text{
            id: waitingForBootloaderDialogText
            text: qsTr("Waiting for bootloader...") + application.emptyString
            wrapMode: Text.WordWrap
            color: main.colorFontLight
            font.bold: false
            font.pixelSize: 50 * main.vScaleFactor
            anchors.horizontalCenter: parent.horizontalCenter
            anchors.verticalCenter: parent.verticalCenter
            z: waitingForBootloaderDialog.z + 1
        }

        Component.onCompleted: hide()
    }

    Timer {
        id: hexPollTimer
        interval: 5000
        repeat: true
        running: root.visible && (root.opacity >0) && !firmwareUpdating
        onTriggered: {
            nucleusfirmwareloader.findHexFile()
        }
    }

    ModalDialog{
        id: firmwareUpdatingDialog
        blockKeys: true

        //visible: bootloader.connected && bootloader.progress > 0
        property alias text: bootLoadStatusText.text

        Item {
            z: firmwareUpdatingDialog.z + 1
            anchors.fill: parent

            Text{
                id: bootLoadStatusText
                text: ""
                wrapMode: Text.WordWrap
                color: main.colorFontLight
                font.bold: false
                font.pixelSize: 50 * main.vScaleFactor
                anchors.horizontalCenter: parent.horizontalCenter
                anchors.bottom: bootloadProgress.top
                anchors.bottomMargin: 5 * main.vScaleFactor
            }

            ProgressBar {
                id: bootloadProgress
                width: 0.5 * parent.width
                height: 0.15 * parent.height
                anchors.horizontalCenter: parent.horizontalCenter
                anchors.verticalCenter: parent.verticalCenter
                anchors.verticalCenterOffset: 0.5*bootLoadStatusText.height
                value: nucleusfirmwareloader.progress
            }
        }
        Component.onCompleted: hide()

    }

    Component.onCompleted: {
        console.log("starting software update check...")
        nucleusfirmwareloader.findHexFile()
    }
}

