import QtQuick 2.12
import StorageMgr 1.0
import Components 1.0

Item {
    id: root

    property int imageIndex: -1
    property bool zoomEffect: false
    property bool fadeEffect: false
    property int slideInterval: 5000
    property ListModel imageList: imageListModel
    property string slidesPath: "/data/slideshow"
    property string slidesUserPath: "/data/slideshow/user"
    property int _zoomDuration: 4000
    property int _fadeDuration: 1500
    property bool useCustomModel: false
    property bool showControls: false
    property bool playing: true
    property int fillMode: Image.PreserveAspectFit

    property bool isMenuDisplayed: false
    property bool isMenuMoving: false
    signal action()

    Component.onCompleted: {
        console.log("starting slideshow")

        if (!root.useCustomModel) {
            // use images stored in the slideshow directory, otherwise, use default images
            var slides = slidesStorage.listFiles(root.slidesUserPath, ["*.jpg", "*.png", "*.gif"]).map(function(p) { return root.slidesUserPath + "/" + p })

            if (slides.length === 0) {
                slides = slidesStorage.listFiles(root.slidesPath, ["*.jpg", "*.png", "*.gif"]).map(function(p) { return root.slidesPath + "/" + p })
            }

            if(slides.length > 0){
                imageListModel.clear()

                for( var i=0 ; i < slides.length ; i++) {
                    imageListModel.append( { file: slides[i] })
                }
            }
        }

        root.imageIndex = -1
        nextImage()
    }

    function doImageChangeEffects() {
        if(root.zoomEffect) {
            newImage.x = Math.floor((Math.random() * 500) - 150);
            newImage.y = Math.floor((Math.random() * 100) - 50);
            newImage.scale = Math.random() * 1.5 + 1.2; // initial scale of new image between 1.2 and 2.7
        }

        if( !root.fadeEffect ) {
            oldImage.opacity = 0;
        } else {
            oldImage.opacity = 1.0;
        }

        newImage.source = imageList.get(root.imageIndex).file;
        fade.start();
    }

    function prevImage() {
        if (imageList.count == 0) {
            return;
        }

        root.imageIndex = (root.imageIndex === 0) ? imageList.count - 1 : root.imageIndex - 1;
        doImageChangeEffects();
    }

    function nextImage() {
        if(imageList.count == 0) {
            return;
        }

        root.imageIndex = (root.imageIndex + 1) % imageList.count;
        doImageChangeEffects();
    }

    ParallelAnimation {
        id: fade

        // scale the new image back to 1.0
        PropertyAnimation{ target: newImage ; properties:"scale" ; to: 1.0; duration: _zoomDuration ; easing.type: Easing.InOutQuad  }
        // fade in the new image from 0 opacity
        SequentialAnimation {
            NumberAnimation { target: newImage; property: "opacity"; duration: _fadeDuration; to: 1.0; easing.type: Easing.InOutQuad }
            NumberAnimation { target: oldImage; property: "opacity"; duration: 1000; to: 0.0; easing.type: Easing.InOutQuad }
        }
        // and slide it in from a random offset.
        SequentialAnimation {
            PropertyAnimation{ target: newImage ; properties:"x,y" ; to: 0 ; duration: _zoomDuration ; easing.type: Easing.InOutQuad  }
            // copy the new image to the old image behind it...
            PropertyAction { target: oldImage; property: "source" ; value: newImage.source }
            PropertyAction { target: oldImage; property: "opacity" ; value: 1.0 }
            // now hide the new image, ready for the next transition
            PropertyAction { target: newImage; property: "opacity" ; value: 0.0 + root.fadeEffect ? 0 : 1.0 }
        }
    }

    Timer {
        id: switchTimer
        interval: root.slideInterval
        running: root.playing
        repeat: true
        onTriggered: nextImage()
    }

    Image {
        id: oldImage
        smooth: (x + y) == 0  // smooth only when image stops at origin (not moving)
        opacity: 1.0
        anchors.fill: parent
        fillMode: root.fillMode
        z: 1    // old image is always under the new image
    }

    Image {
        id: newImage
        smooth: (x + y) == 0
        opacity: 0.0
        width: parent.width
        height: parent.height
        x:0
        y:0
        fillMode: root.fillMode
        z: 2
    }

    Rectangle {
        id: controlsBackground
        visible: root.showControls
        anchors.fill: controls
        anchors.margins: -5 * main.scaleFactor
        z: controls.z - 1
        color: main.colorDialogBackground
        border.color: main.colorUIBorder
        border.width: main.widthBorder
        radius: main.defaultRadius
    }

    Row {
        id: controls
        property int iconWidth: 72 * main.hScaleFactor
        visible: root.showControls
        width: (iconWidth+spacing) * 3
        height: 25 * main.vScaleFactor
        spacing: 5 * main.hScaleFactor
        anchors.bottom: parent.bottom
        anchors.bottomMargin: 10 * main.vScaleFactor
        anchors.horizontalCenter: parent.horizontalCenter
        z: 4

        ScalableImage {
            sourceName: "Bluetooth/RWD_Icon"
            fillMode: Image.PreserveAspectFit
            width: controls.iconWidth
            height: parent.height

            MouseArea {
                anchors.fill: parent
                onClicked: {
                    if (playing) {
                        switchTimer.restart()
                    }

                    root.prevImage()
                    buzzer.buttonPress()
                    root.action()
                }

                Rectangle {
                    anchors.fill: parent
                    color: main.colorGenericStimulusArea
                    opacity: 0.3
                    visible: main.showStimulusArea
                }
            }
        }

        ScalableImage {
            sourceName: root.playing ? "Bluetooth/Pause_Icon" : "Bluetooth/Play_Icon"
            fillMode: Image.PreserveAspectFit
            width: controls.iconWidth
            height: parent.height

            MouseArea {
                anchors.fill: parent
                onClicked: {
                    root.playing = !root.playing
                    buzzer.buttonPress()
                    root.action()
                }

                Rectangle {
                    anchors.fill: parent
                    color: main.colorGenericStimulusArea
                    opacity: 0.3
                    visible: main.showStimulusArea
                }
            }
        }

        ScalableImage {
            sourceName: "Bluetooth/FWD_Icon"
            fillMode: Image.PreserveAspectFit
            width: controls.iconWidth
            height: parent.height

            MouseArea {
                anchors.fill: parent
                onClicked: {
                    if (playing) {
                        switchTimer.restart()
                    }

                    root.nextImage()
                    buzzer.buttonPress()
                    root.action()
                }

                Rectangle {
                    anchors.fill: parent
                    color: main.colorGenericStimulusArea
                    opacity: 0.3
                    visible: main.showStimulusArea
                }
            }
        }
    }

    ListModel {
        id: imageListModel

        ListElement { file: "../graphics/ui3_0/Facility/facilitywindow_1YF_021213.png" }
        ListElement { file: "../graphics/ui3_0/Facility/facilitywindow_2GS_021213.png" }
        ListElement { file: "../graphics/ui3_0/Facility/facilitywindow_3TRU_021213.png" }
        ListElement { file: "../graphics/ui3_0/Facility/facilitywindow_4TRU_021213.png" }
    }

    StorageMgr {
        id: slidesStorage
    }
}
