import QtQuick 2.12
import QtQuick.Layouts 1.12
import Components 1.0
import "../Keypad/" // For Keypad
import WorkoutRunData 1.0
import WorkoutManager 1.0
import "../"
import "../WorkoutSetup"

Item {
    id: root
    
    width: 907 * main.hScaleFactor
    height: 425 * main.vScaleFactor
    readonly property int vMargins: 26 * main.vScaleFactor
    readonly property int hMargins: 27 * main.hScaleFactor

    readonly property int buttonWidth: 188 * main.hScaleFactor
    readonly property int buttonHeight: 111 * main.vScaleFactor
    readonly property int rowSpacing: 20 * main.vScaleFactor
    readonly property int columnSpacing: 24 * main.vScaleFactor
    readonly property int gridWidth: buttonWidth + columnSpacing
    readonly property int gridHeight: buttonHeight + rowSpacing

    /**
     * Property that indicates whether the fan is enabled or not
     *   Used to set fan button's selected property
     */
    property bool fanEnabled: false

    property bool cruiseControlAvailable: heartRateManager.heartRateAvailable && !workoutProgram.isHRCProgram && !workoutProgram.isC360Program && workoutProgram.currentProgram != WorkoutProgram.FITNESS_TEST_PROG && (aWorkoutRunData.mode == WorkoutRunData.WORKOUT_MODE)
    property bool autoStrideAvailable: machineinfo.productTypeString === "Adjustable Stride Elliptical"
    property bool bikeModeAvailable: machineinfo.productTypeString == "Bike"
    property bool isCoolDownMode: workoutManager.workoutMode === WorkoutRunData.COOL_DOWN_MODE

    /**
     * Signal emitted when close button is clicked when in Menu state
     */
    signal close();
    /**
     * Signal emitted when fan button is clicked
     */
    signal fanButtonClicked();
    /**
     * Signal emitted when weight is entered
     */
    signal weightEntered(int weight);
    /**
     * Signal emitted when cruise control button is clicked
     */
    signal cruiseControlButtonClicked();
    /**
     * Signal emitted when auto stride button is clicked
     */
    signal autoStrideButtonClicked();
    /**
     * Signal emitted when bike mode button is clicked
     */
    signal bikeModeButtonClicked();
    
    state: "Menu"
    states: [
        State { 
            name: "Menu"
            PropertyChanges { target: changeWorkoutMenu; visible: false }
            PropertyChanges { target: adjustWorkoutMenu; visible: false }
            PropertyChanges { target: weightEntryMenu; visible: false }
        },
        State { 
            name: "WeightEntry"
            PropertyChanges { target: changeWorkoutMenu; visible: false }
            PropertyChanges { target: adjustWorkoutMenu; visible: false }
            PropertyChanges { target: weightEntryMenu; visible: true }
        },
        State {
            name: "ChangeWorkout"
            PropertyChanges { target: changeWorkoutMenu; visible: true }
            PropertyChanges { target: adjustWorkoutMenu; visible: false }
            PropertyChanges { target: weightEntryMenu; visible: false }
        },
        State {
            name: "AdjustWorkout"
            PropertyChanges { target: changeWorkoutMenu; visible: false }
            PropertyChanges { target: adjustWorkoutMenu; visible: true }
            PropertyChanges { target: weightEntryMenu; visible: false }
        },
        State {
            name: "NewWorkout"
            PropertyChanges { target: changeWorkoutMenu; visible: false }
            PropertyChanges { target: adjustWorkoutMenu; visible: false }
            PropertyChanges { target: weightEntryMenu; visible: false }
        },
        State {
            name: "BluetoothMedia"
            PropertyChanges {
                target: ipod_window
                visible: true
                onClose: {
                    main.showMediaBrowser = false
                    root.state = "Menu"
                }

            }
            PropertyChanges { target: changeWorkoutMenu; visible: false }
            PropertyChanges { target: adjustWorkoutMenu; visible: false }
            PropertyChanges { target: weightEntryMenu; visible: false }
            StateChangeScript { script: ipod_window.actionConnectionType("audio") }
        },
        State { 
            name: "BluetoothHeartRate"
            PropertyChanges {
                target: ipod_window
                visible: true
                onClose: {
                    main.showMediaBrowser = false
                    root.state = "Menu"
                }
            }
            PropertyChanges { target: changeWorkoutMenu; visible: false }
            PropertyChanges { target: adjustWorkoutMenu; visible: false }
            PropertyChanges { target: weightEntryMenu; visible: false }
            StateChangeScript { script: ipod_window.actionConnectionType("hr") }
        },
        State { 
            name: "BluetoothHeadset"
            PropertyChanges {
                target: ipod_window
                visible: true
                onClose: {
                    main.showMediaBrowser = false
                    root.state = "Menu"
                    root.close()
                }
            }
            PropertyChanges { target: changeWorkoutMenu; visible: false }
            PropertyChanges { target: adjustWorkoutMenu; visible: false }
            PropertyChanges { target: weightEntryMenu; visible: false }
            StateChangeScript { script: ipod_window.actionConnectionType("ear") }
        }
    ]
    
    Component.onCompleted: reset();
    
    /**
     * Resets menu to its initial state
     */
    function reset() {
        state = "Menu";
    }
    
    function updateAdjustModel() {
        adjustWorkoutMenuParams.updateModel()
    }

    Connections {
        target: bluetoothLEAdapter
        onHrConnectedChanged: {
            if(bluetoothLEAdapter.hrConnected === true && root.state === "BluetoothHeartRate") {
                root.state = "Menu"
            }
        }

        onHrDisconnectingComplete: {
            if(root.state === "BluetoothHeartRate") {
                root.state = "Menu"
            }
        }
    }

    Rectangle {
        id: background
        anchors.fill: root
        color: main.colorButtonBackground
        border.color: main.colorUIBorder
        border.width: main.widthBorder

        z: menuLayout.z-1
        
        // Mouse area to intercept mouse presses, so they do not go to items behind the menu
        MouseArea {
            anchors.fill: parent
        }
    }

    CloseButton { // x

        anchors {
            left: parent.left
            leftMargin: 98 * main.hScaleFactor
            bottom: parent.bottom
            bottomMargin: 27 * main.vScaleFactor
        }

        visible: root.state === "Menu" || root.state === "WeightEntry" || root.state === "ChangeWorkout" || root.state === "AdjustWorkout"

        onClicked: {
            buzzer.buttonPress()
            if(root.state === "Menu") {
                root.close();
            } else {
                root.state = "Menu";
            }
        }
    }

    Item {
        id: menuLayout

        anchors.fill: root
        anchors.topMargin: root.vMargins
        anchors.leftMargin: root.hMargins
        anchors.rightMargin: root.hMargins
        anchors.bottomMargin: root.vMargins
        
    /*******************************
     *     0     1     2   3   4
     * 0 [ A ] [ B ] [ C ] | [ D ]
     *                     |
     * 1 [ E ] [ F ] [ G ] | [ H ]
     *                     | 
     * 2  [X]              | [ I ]
     *******************************/
        
        StandardButton { // A
            id: weightButton
            visible: (root.state === "Menu") || (root.state === "WeightEntry");
            enabled: (root.state === "Menu") || (root.state === "WeightEntry");
            selected: (root.state === "WeightEntry");
            
            onClicked: root.state = "WeightEntry"
            
            buttonText: qsTr("ENTER WEIGHT") + application.emptyString

            icon.sourceName: "ToolsMenu/keypad_icon"
            
            anchors {
                left: parent.left
                leftMargin: 0 * gridWidth
                top: parent.top
                topMargin: 0 * gridHeight
            }
        }
        
        StandardButton { // B
            id: fanButton
            visible: (root.state === "Menu");
            enabled: (root.state === "Menu");
            selected: root.fanEnabled;
            
            buttonText: qsTr("FAN") + application.emptyString

            icon.sourceName: "ToolsMenu/fan_icon"
            
            onClicked: root.fanButtonClicked()

            anchors {
                left: parent.left
                leftMargin: 1 * gridWidth
                top: parent.top
                topMargin: 0 * gridHeight
            }
        }
        
        StandardButton { // C
            id: changeWorkoutButton
            visible: root.state === "Menu" || root.state === "ChangeWorkout"
            enabled: root.state === "Menu"
            selected: root.state === "ChangeWorkout"
            
            buttonText: qsTr("CHANGE WORKOUT") + application.emptyString

            icon.sourceName: "ToolsMenu/change_workout_icon"
            
            onClicked: root.state = "ChangeWorkout"

            anchors {
                left: parent.left
                leftMargin: 2 * gridWidth
                top: parent.top
                topMargin: 0 * gridHeight
            }
        }
        
        StandardButton { // D
            id: btMediaButton
            visible: (root.state === "Menu")
            enabled: (root.state === "Menu" && bluetoothAdapter.connectedHeadset === false)
            selected: (root.state === "BluetoothMedia");
            
            buttonText: (bluetoothAdapter.connected ? qsTr("CONNECTED") : qsTr("CONNECT")) + application.emptyString

            icon.sourceName: "Bluetooth/Bluetooth_Media_icon"
            
            onClicked: root.state = "BluetoothMedia";

            anchors {
                left: parent.left
                leftMargin: 3 * gridWidth + separator.width + columnSpacing
                top: parent.top
                topMargin: 0 * gridHeight
            }
        }

        StandardButton { // E - Cruise Control
            id: cruiseControlButton
            visible: root.cruiseControlAvailable && (root.state === "Menu")
            enabled: root.cruiseControlAvailable && (root.state === "Menu")
            selected: workoutManager.cruiseControlEnabled

            buttonText: qsTr("CRUISE CONTROL") + application.emptyString

            icon.sourceName: "WF/WF_Button_HRC"

            onClicked: root.cruiseControlButtonClicked()

            anchors {
                left: parent.left
                leftMargin: 0 * gridWidth
                top: parent.top
                topMargin: 1 * gridHeight
            }
        }

        StandardButton { // F - Auto Stride
            id: autoStrideButton
            visible: root.autoStrideAvailable && (root.state === "Menu")
            enabled: root.autoStrideAvailable && (root.state === "Menu")
            selected: machinecontrol.autoStrideEnabled

            buttonText: qsTr("AUTO STRIDE") + application.emptyString

            icon.sourceName: "AdjStride/Auto_Stride_Icon"

            onClicked: root.autoStrideButtonClicked()

            anchors {
                left: parent.left
                leftMargin: 1 * gridWidth
                top: parent.top
                topMargin: 1 * gridHeight
            }
        }

        StandardButton { // F - Bike Mode
            id: bikeModeButton
            visible: root.bikeModeAvailable && (root.state === "Menu") && !workoutScreen.bikeWattsModeRequired
            enabled: root.bikeModeAvailable && (root.state === "Menu") && !workoutScreen.bikeWattsModeRequired
            selected: workoutScreen.bikeWattsModeEnabled

            buttonText: qsTr("Constant Watts Mode") + application.emptyString

            icon.sourceName: "BikeMode/Bike_Mode_Icon"

            onClicked: root.bikeModeButtonClicked()

            anchors {
                left: parent.left
                leftMargin: 1 * gridWidth
                top: parent.top
                topMargin: 1 * gridHeight
            }
        }

        StandardButton { // G - Units Mode
            id: unitsModeButton
            selected: workoutScreen.unitState === "METRIC"
            visible: root.state === "Menu"
            enabled: root.state === "Menu"

            buttonText: selected ? qsTr("Metric" ,"Units") + application.emptyString : qsTr("Imperial", "Units") + application.emptyString

            onClicked: {
                selected = !selected
                workoutScreen.setUnitState(selected)
            }

            anchors {
                left: parent.left
                leftMargin: 2 * gridWidth
                top: parent.top
                topMargin: 1 * gridHeight
            }

            onVisibleChanged: selected = workoutScreen.unitState === "METRIC"
        }

        StandardButton { // H
            id: btHeartRateButton
            visible: root.state === "Menu"
            enabled: root.state === "Menu"
            selected: (root.state === "BluetoothHeartRate");
            
            buttonText: (bluetoothLEAdapter.hrConnected ? qsTr("DISCONNECT") : qsTr("CONNECT")) + application.emptyString

            icon.sourceName: "Bluetooth/Bluetooth_HRicon"
            
            onClicked: {
                root.state = "BluetoothHeartRate";
                if(bluetoothLEAdapter.hrConnected) {
                    bluetoothLEAdapter.disconnect()
                }
            }

            anchors {
                left: parent.left
                leftMargin: 3 * gridWidth + separator.width + columnSpacing
                top: parent.top
                topMargin: 1 * gridHeight
            }
        }
        
        StandardButton { // I
            id: btHeadsetButton
            visible: (root.state === "Menu")
            enabled: (root.state === "Menu" && bluetoothAdapter.connected === false)
            selected: (root.state === "BluetoothHeadset")
            
            buttonText: (bluetoothAdapter.connectedHeadset ? qsTr("DISCONNECT") : qsTr("CONNECT")) + application.emptyString

            icon.sourceName: "Bluetooth/Bluetooth_Headphones_Icon"
            
            onClicked: {
                root.state = "BluetoothHeadset";
                if(bluetoothAdapter.connectedHeadset )
                    bluetoothAdapter.disconnectHeadset()
            }

            anchors {
                left: parent.left
                leftMargin: 3 * gridWidth + separator.width + columnSpacing
                top: parent.top
                topMargin: 2 * gridHeight
            }
        }
    }

    ScalableImage { // |
        id: separator
        visible: (root.state === "Menu");
        enabled: (root.state === "Menu");

        fillMode: Image.TileVertically
        horizontalAlignment: Image.AlignHCenter
        verticalAlignment: Image.AlignTop

        width: 5
        height: menuLayout.height
        anchors {
            left: parent.left
            leftMargin: 3 * gridWidth + root.hMargins
            top: parent.top
            topMargin: root.vMargins
        }
        sourceName: "Components/dotted_line_separator_single";
    }
        
    Item {
        id: weightEntryMenu
        visible: false
        anchors.verticalCenter: parent.verticalCenter
        anchors.right: parent.right
        anchors.rightMargin: 10 * main.hScaleFactor
        width: parent.width * 0.4
        height: parent.height * 0.9
        Keypad {
            id: weightKeypad
            anchors.centerIn: parent
            scaleFactor: 0.95
            keyWidth: 65 * main.hScaleFactor
            color: "transparent"

            units: workoutScreen.unitState === "METRIC" ? qsTr("KG") : qsTr("LBS")

            allow_decimal: false
            allow_close: false
            show_entry: true

            length: 3
            decimalPlaces: 0
            instructions: qsTr("ENTER WEIGHT") + application.emptyString

            onValueEntered: {
                if(val.length > 0) root.weightEntered(Number(val));
            }

            onVisibleChanged: {
                if(visible) {
                    var w = workoutScreen.unitState === "METRIC" ? Globals.poundToKg(aWorkoutRunData.userWeight) : aWorkoutRunData.userWeight
                    setValue(w)
                }
            }

            Connections {
                target: workoutScreen
                onUnitStateChanged: {
                    weightKeypad.units = workoutScreen.unitState === "METRIC" ? qsTr("KG") : qsTr("LBS")
                }
            }

            // re-translate the strings when the language changes
            Connections {
                target: application
                onLanguageChanged: {
                    weightKeypad.instructions = qsTr("ENTER WEIGHT") + application.emptyString
                    weightKeypad.units = workoutScreen.unitState === "METRIC" ? qsTr("KG") : qsTr("LBS")
                }
            }
        }
    }

    Item {
        id: changeWorkoutMenu
        visible: false

        anchors {
            top: parent.top
            topMargin: 160 * main.vScaleFactor
            bottom: menuLayout.bottom
            left: parent.left
            leftMargin: 200 * main.hScaleFactor
            right: parent.right
            rightMargin: 80 * main.hScaleFactor
        }

        Grid {
            anchors.fill: parent
            rows: 2
            columns: 2
            columnSpacing: 10 * main.hScaleFactor
            rowSpacing: 10 * main.hScaleFactor
            verticalItemAlignment: Grid.AlignVCenter

            property int textWidth: width - adjustWorkoutButton.width
            property int buttonHeight: (height - rowSpacing)/2

            StandardButton {
                id: adjustWorkoutButton

                enabled: workoutProgram.programAdjustable && !root.isCoolDownMode
                height: parent.buttonHeight
                buttonText: qsTr("CHANGE WORKOUT") + application.emptyString

                icon.sourceName: "ToolsMenu/change_workout_icon"

                onClicked: root.state = "AdjustWorkout"
            }

            Text {
                id: changeLabel
                width: parent.textWidth
                color: main.colorFontLight
                font.pixelSize: 12 * main.vScaleFactor
                wrapMode: Text.WordWrap
                text: qsTr(changeText)
                horizontalAlignment: main.rightToLeft ? Text.AlignRight : Text.AlignLeft
                property string changeText: workoutProgram.programAdjustable && !root.isCoolDownMode ?
                                            QT_TR_NOOP("<h2>Would you like to change this workout?</h2><ul><li>Workout will continue</li><li>Add or removed time or change intensity or level of this workout</li></ul>" ) :
                                            QT_TR_NOOP("<h2>Would you like to change this workout?</h2><ul><li>Sorry, this workout does not allow changes once the workout has begun</li></ul>")

                // re-translate the strings when the language changes
                Connections {
                    target: application
                    onLanguageChanged: { changeLabel.text = qsTr(changeLabel.changeText) }
                }

                Connections {
                    target: workoutProgram
                    onProgramAdjustableChanged: { changeLabel.text = qsTr(changeLabel.changeText) }
                }
            }

            StandardButton {
                id: newWorkoutButton

                visible: false // Hide functionality for now
                enabled: !root.isCoolDownMode
                height: parent.buttonHeight
                buttonText: qsTr("NEW WORKOUT") + application.emptyString

                icon.sourceName: "HomeScreen/WorkoutFinderIcon"

                onClicked: {
                    main.isNewWorkoutSelected = true
                    root.state = "NewWorkout"
                    workoutScreen.stopWorkout();
                    main.state = "MenuState"
                }
            }

            Text {
                id: diffLabel
                visible: false // Hide functionality for now
                width: parent.textWidth
                color: main.colorFontLight
                font.pixelSize: 12 * main.vScaleFactor
                wrapMode: Text.WordWrap
                text: qsTr(diffText)
                horizontalAlignment: main.rightToLeft ? Text.AlignRight : Text.AlignLeft

                property string diffText: QT_TR_NOOP("<h2>Would you like to choose a different workout?</h2><ul><li>This workout will end and you may choose a different workout</li><li>TIME, CALORIES, DISTANCE will be added to next workout</li></ul>")

                // re-translate the strings when the language changes
                Connections {
                    target: application
                    onLanguageChanged: { diffLabel.text = qsTr(diffLabel.diffText) }
                }
            }
        }
    }

    Item {
        id: adjustWorkoutMenu
        visible: false
        anchors.fill: parent
        anchors.margins: 40 * main.scaleFactor
        anchors.topMargin: 10 * main.scaleFactor
        property string programNameTranslated: ""

        onVisibleChanged: {
            programNameTranslated = qsTranslate("Menu", aWorkoutRunData.programName) + application.emptyString
        }

        // Title Text
        TextLabel {
            id: program_name
            anchors.left: parent.left
            anchors.top: parent.top
            anchors.topMargin: 10 * main.vScaleFactor
            anchors.right: parent.right
            text: parent.programNameTranslated

            font.pixelSize: 20 * main.vScaleFactor
        }

        SetupParams {
            id: adjustWorkoutMenuParams
            visible: true
            anchors {
                top: program_name.bottom
                bottom: parent.bottom
                left: parent.left
                right: parent.right
            }

            isAdjustWorkout: true
            model: workoutProgram.normalProgramModel
        }

        StandardButton {
            id: nextButtonTools

            anchors {
                left: parent.left
                leftMargin: 450 * main.hScaleFactor
                bottom: parent.bottom
            }

            width: 76 * main.hScaleFactor
            height: 57 * main.vScaleFactor
            sourceName: "HomeScreen/QuickStartIcon"
            onClicked: {
                buzzer.buttonPress()
                adjustWorkoutMenuParams.updateBaseModel();
                if(root.state === "Menu") {
                    root.close();
                } else {
                    root.state = "Menu";
                }
            }
        }
    }
}
