import QtQuick 2.12
import WorkoutManager 1.0
import "../string_format.js" as Format
import Components 1.0
import QtQuick.Layouts 1.12
import ".."

Item {
    id: mainProgItem
    width: main.width
    height: main.height

    property int selectedIndex: 0
    property bool valueEntered: false
    property int fieldIndex: 1
    property string programName: ""
    property string programNameTranslated: qsTranslate("Menu", programName) + application.emptyString
    property string programFileName: ""
    property int programIndex: workoutProgram.programIndex
    property int targetHRType: targetHRTypeEnum.invalid
    property int progSetupStep: 1
    property string profile1Title: ""
    property string profile2Title: ""
    property string profile1Units: ""
    property string profile2Units: ""
    property int workTime: 0
    property int segTime: 0
    property int numSegs: 10
    property int paceLevelForCustom: 1
    property int profile2Max: 1
    property int workloadLevelForCustom: 1
    property int profile1Max: 1
    property int ageForProgSummary: 35
    property int weightForProgSummary: 155
    property bool c360TypeProg: workoutProgram.isC360Program
    property bool isC360Custom: programIndex == WorkoutProgram.CUSTOM_C360_PROG
    property bool isSegmentTypeProg: !c360TypeProg
    property int customProgType: workoutProgram.customProgType
    property variant c360SetupModel: workoutProgram.c360Program.c360ProgramModel
    property variant setupParamList: workoutProgram.normalProgramModel
    property variant savedWorkouts: saveProgLoader.item
    property int startSpeedMinWarning: main.isStepmill ? 54 : 40    // warn if starting tread > x mph
    property variant basicSetup: setupParamsLoader.item
    property bool saveHasHappened: false
    property int saveCount: 0
    property bool isElevationType: determineElevationType()
    property alias targetHRTypeEnum: targetHeartRateTypeEnum
    property string previousState: "Simple"

    signal startProgram()
    signal showHRChart()
    signal saveCustomProgram(string programFileName, int saveLocation)
    signal dataItemChanged()

    QtObject{
        id: targetHeartRateTypeEnum
        property int invalid: 0
        property int aerobic: 1
        property int target: 2
    }

    function doStart() {
        var correctionsMade = 0;

        if (setupScreenManager.state === "C360Step2") {
            correctionsMade = loaderC360.item.validate();
        } else {
            correctionsMade = basicSetup.validateEntries();
        }

        // check if a setting was corrected by validation.  If so, don't start.
        if(correctionsMade) {
            return;
        }

        // check if we need a starting speed confirmation for treadmill
        var firstSpeed = workoutProgram.profile2.get(0);
        if((main.isTreadmill || main.isStepmill) && workoutProgram.profile2.isCustom && (firstSpeed > mainProgItem.startSpeedMinWarning) ) {
            customSpeedWarningConfirmation.prompt = qsTr("BEGINNING SPEED:") + "\n" + speedString(firstSpeed);
            customSpeedWarningConfirmation.show();

            if (options.workoutSetupTimeout > 0) {
                customSpeedWarningConfirmation.startTimeout(options.workoutSetupTimeout);
            }
        } else if (workoutProgram.programIndex === WorkoutProgram.TREAD_GERKIN_TEST_PROG ||
                   workoutProgram.programIndex === WorkoutProgram.STEP_WFI_TEST_PROG) {
            gerkinTestPrompt.show();
        } else {
            console.debug("main.startpr!!!!!!")
            mainProgItem.startProgram();
        }
    }

    function nextState() {
        basicSetup.validateEntries();

        switch(setupScreenManager.state) {
            case "Step1":
                if(isC360Custom) {
                    setupScreenManager.state = "C360Step2";
                } else {
                    setupScreenManager.state = "Step2";
                }

                break;
            case "Step2":
                setupScreenManager.state = "Step3";
                break;
            case "Step3":   // shouldn't happen, next button hidden in step3
                setupScreenManager.state = "Step3";
                print("error, next button in Step3!");
                break;
        }
    }

    function setProgramDetails (item) {
        mainProgItem.programName = item.label;
        var tmpStr = item.program.split("/")
        tmpStr = tmpStr[1].split('.')
        tmpStr = tmpStr[0]
        mainProgItem.programFileName = tmpStr;
    }

    function clearListView() {
        mainProgItem.selectedIndex = 0;
        mainProgItem.valueEntered = false;
        mainProgItem.fieldIndex = 1;
        mainProgItem.programName = "";
        mainProgItem.programFileName = "";
        mainProgItem.targetHRType = targetHRTypeEnum.invalid;
        mainProgItem.progSetupStep = 1;
        mainProgItem.workTime = 0;
        mainProgItem.segTime = 0;
        mainProgItem.numSegs = 1;
        mainProgItem.paceLevelForCustom =  1;
        mainProgItem.workloadLevelForCustom = 1;
        mainProgItem.profile2Max = 1;
        mainProgItem.profile1Max = 1;
        mainProgItem.ageForProgSummary = 35;
        mainProgItem.weightForProgSummary = 155;
        mainProgItem.saveHasHappened = false;
        mainProgItem.saveCount = 0;
    }

    function startQuickStart (element) {
        mainProgItem.startProgram();
    }

    onSetupParamListChanged: mainProgItem.init()

    // re-translate the strings when the language changes
    Connections {
        target: application
        onLanguageChanged: { mainProgItem.updateStrings() }
    }

    function updateStrings() {
        // Page 2's Graph Setup Text
        if(machineinfo.productTypeString == "Treadmill"  || main.isStepmill) {
            mainProgItem.profile1Title = qsTr("INCLINE") + application.emptyString;
            mainProgItem.profile1Units = qsTr("%") + application.emptyString;
            mainProgItem.profile2Title = qsTr("SPEED") + application.emptyString;
            mainProgItem.profile2Units = main.isStepmill ? qsTr("SPM") : (options.metricUnits ? qsTr("KPH") : qsTr("MPH"));
        } else {
            mainProgItem.profile1Title = qsTr("WORKLOAD") + application.emptyString;
            mainProgItem.profile1Units = (options.en957Enabled ? qsTr("WORK") : qsTr("WATTS")) + application.emptyString;
            mainProgItem.profile2Title = qsTr("PACE") + application.emptyString;
            mainProgItem.profile2Units = qsTr("RPM") + application.emptyString;
        }
    }

    function init() {
        hrGuide.hide();
        gerkinTestPrompt.hide();
        var program = programIndex;

        if(programIndex == WorkoutProgram.NO_PROG) {
            return;
        }

        mainProgItem.updateStrings();

        if( isC360Custom ) {
            setupScreenManager.state = "Step1";
        } else if( workoutProgram.profile1.isCustom || workoutProgram.profile2.isCustom ) {
            if( workoutProgram.currentProgram == WorkoutProgram.SAVED_WORKOUT ) {
                setupScreenManager.state = "Step3";
            }
            else {
                setupProfileEditor();
                setupScreenManager.state = "Step1";
            }
        } else {
            setupScreenManager.state = "Simple";
        }
    }

    onActiveFocusChanged: {
        if(!activeFocus) {
            gerkinTestPrompt.hide()
        }
    }

    onWorkTimeChanged: {
        mainProgItem.segTime = workTime*60 / mainProgItem.numSegs
    }

    function setNumSegments(n) {
        workoutProgram.profile1.count = n;
        workoutProgram.profile2.count = n;
        mainProgItem.numSegs = n;
        mainProgItem.segTime = workTime*60 / mainProgItem.numSegs;
        basicSetup.setParameter(WorkoutProgram.SEGMENT_TYPE, n);
        basicSetup.recalcMaxSegments();
    }

    function setupProfileEditor() {
        var workoutType = mainProgItem.programIndex;

        if(workoutType === WorkoutProgram.SAVED_WORKOUT) {
            workoutType = mainProgItem.customProgType;
            return;
        } else {
            setNumSegments(mainProgItem.numSegs);
        }
    }

    function computeProfileStats() {
       mainProgItem.profile1Max = workoutProgram.profile1.findMax("segmentLevel");
       mainProgItem.profile2Max = workoutProgram.profile2.findMax("segmentLevel");
    }

    // return a string with speed converted to appropriate units with units label (e.g. 103 = "10.3 MPH")
    function speedString(mphTimesTen) {
        if(main.isStepmill)
            return mphTimesTen + " " + qsTr("SPM")

        var value;
        var units;

        if(options.metricUnits) {
            value = Globals.mileToKm(mphTimesTen / 10.0).toFixed(1);
            units = qsTr("KMH");
        } else {
            value = mphTimesTen / 10.0;
            units = qsTr("MPH");
        }

        return value + " " + units;
    }

    function determineElevationType () {
        if(setupParamList == null) {
            return false
        }

        if(main.isAlpineRunner || main.isStepmill)
        {
            var elevItem = setupParamList.find("fieldType", WorkoutProgram.ELEVATION_TYPE)
            if(!elevItem) {
                return false;
            }
            return true;
        }
        return false;
    }

    function determineBackState() {
        if(setupScreenManager.state === "Step2") {
            basicSetup.validateEntries()
            step1Button.selected = true
            step2Button.selected = false
            step3Button.selected = false
            setupScreenManager.state = "Step1"
            return "SetupState"
        }
        else if(setupScreenManager.state === "Step3") {
            basicSetup.validateEntries()
            step1Button.selected = false
            step2Button.selected = true
            step3Button.selected = false
            setupScreenManager.state =  "Step2"
            return "SetupState"
        }
        else if(setupScreenManager.state === "SaveWorkout") {
            setupScreenManager.state = previousState
            return "SetupState"
        }
        else {
            main.showMediaBrowser = false
            return "MenuState"
        }
    }

    Item {
        id: invisibleBackArea
        height: parent.height
        width: 120 * main.hScaleFactor
        anchors {
            left: parent.left
            top: parent.top
        }

        MouseArea {
            anchors.fill: parent
            onClicked: backButton.clicked();

            Rectangle {
                anchors.fill: parent
                color: main.colorGenericStimulusArea
                opacity: 0.3
                visible: main.showStimulusArea
            }
        }
    }

    BackButton {
        id: backButton
        z: invisibleBackArea.z + 1
        backState: "MenuState";
        onClicked: {
            main.state = determineBackState();
            buzzer.buttonPress()
        }
        anchors.bottomMargin: 125 * main.vScaleFactor
    }

    ColumnLayout {
        anchors {
            left: parent.left
            leftMargin: 1095 * main.hScaleFactor
            top: parent.top
            topMargin: 43 * main.vScaleFactor
            right: parent.right
            rightMargin: 72 * main.hScaleFactor
            bottom: parent.bottom
            bottomMargin: 575 * main.vScaleFactor
        }
        spacing: 5 * main.vScaleFactor
        enabled: (main.loginAvailable && !userData.loggedIn)
        visible: (main.loginAvailable && !userData.loggedIn)

        Text {
            id: speedUpWorkoutLabel
            Layout.alignment: Qt.AlignCenter
            Layout.preferredWidth: 230 * main.hScaleFactor
            horizontalAlignment: Text.AlignHCenter
            color: main.colorFontGreen
            text: qsTr("SPEED UP YOUR WORKOUT!")
            font.pixelSize: 14 * main.vScaleFactor
            font.bold: false
            wrapMode: Text.WordWrap
        }

        Text {
            id: registerLabel
            Layout.alignment: Qt.AlignCenter
            Layout.preferredWidth: 230 * main.hScaleFactor
            horizontalAlignment: Text.AlignHCenter
            color: main.colorFontLight
            text: qsTr("Register today to make your workout setup easier!")
            font.pixelSize: 14 * main.vScaleFactor
            font.bold: false
            wrapMode: Text.WordWrap
        }

        StandardButton {
            id: signInRegisterButton
            Layout.alignment: Qt.AlignCenter
            width: 182 * main.hScaleFactor
            height: 93 * main.vScaleFactor
            buttonText: qsTr("SIGN IN / REGISTER") + application.emptyString
            onClicked: main.state = "LoginState"

            icon.sourceName: "HomeScreen/SignInRegisterIcon"
        }
    }

    ColumnLayout {
        anchors {
            left: parent.left
            leftMargin: 1095 * main.hScaleFactor
            top: parent.top
            topMargin: 43 * main.vScaleFactor
            right: parent.right
            rightMargin: 72 * main.hScaleFactor
            bottom: parent.bottom
            bottomMargin: 575 * main.vScaleFactor
        }
        spacing: 5 * main.vScaleFactor
        enabled: (main.loginAvailable && userData.loggedIn)
        visible: (main.loginAvailable && userData.loggedIn)

        Text {
            id: welcomeBackLabel
            Layout.alignment: Qt.AlignCenter
            Layout.preferredWidth: 230 * main.hScaleFactor
            horizontalAlignment: Text.AlignHCenter
            color: main.colorFontGreen
            text: qsTr("Welcome Back ") + userData.userName + "!" + application.emptyString
            font.pixelSize: 18 * main.vScaleFactor
            font.bold: false
            wrapMode: Text.WordWrap
        }

        StandardButton {
            id: signOutButton
            Layout.alignment: Qt.AlignCenter
            width: 182 * main.hScaleFactor
            height: 93 * main.vScaleFactor
            buttonText: qsTr("NOT YOU? / LOG OUT") + application.emptyString
            onClicked: main.state = "LoginState"

            icon.sourceName: "HomeScreen/SignInRegisterIcon"
        }
    }

    Item {
        id: setupScreenManager
        anchors.fill: parent
        state: "Simple"

        Loader {    // Page 1 - Basic Parameters
            id: setupParamsLoader
            anchors.fill: parent
            sourceComponent: setupParamsComponent

            Component {
                id: setupParamsComponent

                SetupParams {
                    visible: true
                    model: setupParamList
                    isElevationType: mainProgItem.isElevationType
                }
            }
        }

        Item {      // Page 2 - Custom C360 or Custom Profiles
            id: step2
            visible: false
            anchors.fill: parent

            property variant segmentEditorRef: loaderSegmentEditor.item
            property variant c360ContentRef: loaderC360.item

            Loader {
                id: loaderSegmentEditor

                property bool editorLoaded: (main.state == "SetupState") && (setupScreenManager.state === "Step2")

                sourceComponent: editorLoaded ? segmentEditorComponent : undefined
                anchors.fill: parent

                Component {
                    id: segmentEditorComponent

                    SetupCustomProfile {
                        id: segmentEditor
                        model1: workoutProgram.profile1
                        model2: workoutProgram.profile2
                        model: setupParamList
                        workoutTime: mainProgItem.workTime
                        onWorkoutTimeChanged: {
                            if( mainProgItem.workTime != workoutTime) {
                                mainProgItem.workTime = workoutTime
                                basicSetup.setParameter(WorkoutProgram.TIME_TYPE, mainProgItem.workTime)
                            }
                        }
                        segmentTime: mainProgItem.segTime
                        visible: isSegmentTypeProg && step2.visible
                        anchors.fill: parent
                    }
                }
            }

            Loader {
                id: loaderC360
                sourceComponent: setupScreenManager.state === "C360Step2" ? c360ContentComponent : undefined
                anchors.fill: parent

                Component {
                    id: c360ContentComponent

                    SetupCustomC360 {
                        id: c360Content
                        anchors {
                            fill: parent
                            leftMargin: 0.25 * width
                        }

                        visible: isC360Custom
                        model: c360SetupModel
                    }
                }
            }
        }

        Item {      // Page 3
            id: step3
            visible: false
            anchors.fill: parent
            anchors.topMargin: 10 * main.vScaleFactor

            Loader {
                id: summaryGraph

                property bool summaryLoaded: (main.state == "SetupState") && (setupScreenManager.state == "Step3")

                sourceComponent: summaryLoaded ? summaryComponent : undefined

                Component {
                    id: summaryComponent

                    SetupSummary {
                        id: summaryGraph2
                        profile1Title: mainProgItem.profile1Title
                        profile2Title: mainProgItem.profile2Title
                        profile1model: workoutProgram.profile1
                        profile2model: workoutProgram.profile2
                    }
                }
            }

            // Part 3's Title Text
            TextLabel {
                id: program_name_header
                width: 0.4*main.width
                height: 0.1*main.height
                wrapMode: Text.WordWrap
                horizontalAlignment: Text.AlignLeft
                x: 0.04*main.screenWidth
                y: 47 * main.vScaleFactor
                text: programNameTranslated + " " + qsTr("SUMMARY") + application.emptyString
                font.pixelSize: text.length > 20 * main.vScaleFactor ? (20 * main.vScaleFactor) : (26 * main.vScaleFactor)
            }

            SetupSynopsis {
                id: customWorkoutSynopsis
                profile1Title: mainProgItem.profile1Title
                profile2Title: mainProgItem.profile2Title
                setupAge: ageForProgSummary
                setupWeight: weightForProgSummary
                totalTime: workTime
                numSegments: numSegs
                segmentTime: segTime
                profile1Max: mainProgItem.profile1Max
                profile2Max: mainProgItem.profile2Max
            }
        }

        Loader {
            id: saveProgLoader
            anchors.fill: parent
            anchors.leftMargin: 260* main.hScaleFactor
            anchors.rightMargin: 260* main.hScaleFactor
            sourceComponent: saveProgComponent

            Component {
                id: saveProgComponent
                SavePrograms {
                    id: saveWorkoutItem
                    visible: true
                    programFileToSave: mainProgItem.programFileName
                    onSaveProgram: {
                        basicSetup.validateEntries()
                        workoutProgram.setupComplete() // update program from setup screens before saving
                        saveCustomProgram(programFileName, saveLocation)
                        mainProgItem.saveHasHappened = true
                        mainProgItem.saveCount++
                        saveWorkoutItem.visible=false
                        setupScreenManager.state = mainProgItem.previousState
                    }

                    Connections {
                        target: saveProgLoader

                        onVisibleChanged: saveWorkoutItem.visible = saveProgLoader.visible
                    }
                }
            }
        }

        states: [
            State {
                name: "Simple"
                PropertyChanges { target: setupParamsLoader;    visible: true; }
                PropertyChanges { target: stepButtons;          visible: false; }
                PropertyChanges { target: step1Button;          selected: false; }
                PropertyChanges { target: step2Button;          selected: false; }
                PropertyChanges { target: step3Button;          selected: false; }
                PropertyChanges { target: step2;                visible: false; }
                PropertyChanges { target: step3;                visible: false; }
                PropertyChanges { target: nextButton;           visible: false; }
                PropertyChanges { target: startButton;          visible: true }
                PropertyChanges { target: saveButton;           visible: (options.saveWorkoutEnabled); }
                PropertyChanges { target: saveProgLoader;       visible: false;}
            },
            State {
                name: "Step1"
                PropertyChanges { target: setupParamsLoader;    visible: true; }
                PropertyChanges { target: stepButtons;          visible: !isC360Custom; }
                PropertyChanges { target: step1Button;          selected: true; }
                PropertyChanges { target: step2Button;          selected: false; }
                PropertyChanges { target: step3Button;          selected: false; }
                PropertyChanges { target: step2;                visible: false; }
                PropertyChanges { target: step3;                visible: false; }
                PropertyChanges { target: nextButton;           visible: true; }
                PropertyChanges { target: startButton;          visible: false; }
                PropertyChanges { target: saveButton;           visible: false; }
                PropertyChanges { target: saveProgLoader;       visible: false; }
            },
            State {
                name: "C360Step2"
                PropertyChanges { target: setupParamsLoader;    visible: false; }
                PropertyChanges { target: step2;                visible: true; }
                PropertyChanges { target: step3;                visible: false; }
                PropertyChanges { target: saveButton;           visible: (options.saveWorkoutEnabled); }
                PropertyChanges { target: stepButtons;          visible: false; }
                PropertyChanges { target: nextButton;           visible: false; }
                PropertyChanges { target: startButton;          visible: true; }
                PropertyChanges { target: saveProgLoader;       visible: false; }
            },
            State {
                name: "Step2"
                PropertyChanges { target: setupParamsLoader;    visible: false; }
                PropertyChanges { target: stepButtons;          visible: true; }
                PropertyChanges { target: step1Button;          selected: false; }
                PropertyChanges { target: step2Button;          selected: true; }
                PropertyChanges { target: step3Button;          selected: false; }
                PropertyChanges { target: step2;                visible: true; }
                PropertyChanges { target: step3;                visible: false; }
                PropertyChanges { target: nextButton;           visible: true; }
                PropertyChanges { target: startButton;          visible: false; }
                PropertyChanges { target: saveButton;           visible: false; }
                PropertyChanges { target: saveProgLoader;       visible: false; }
                StateChangeScript { script: { workoutProgram.setupComplete();   setupProfileEditor(); } }
            },
            State {
                name: "Step3"
                PropertyChanges { target: setupParamsLoader;    visible: false; }
                PropertyChanges { target: stepButtons;          visible: true; }
                PropertyChanges { target: step1Button;          selected: false; }
                PropertyChanges { target: step2Button;          selected: false; }
                PropertyChanges { target: step3Button;          selected: true; }
                PropertyChanges { target: step2;                visible: false; }
                PropertyChanges { target: step3;                visible: true; }
                PropertyChanges { target: nextButton;           visible: false; }
                PropertyChanges { target: startButton;          visible: true; }
                PropertyChanges { target: saveButton;           visible: (options.saveWorkoutEnabled); }
                PropertyChanges { target: saveProgLoader;       visible: false; }
                StateChangeScript { script: { workoutProgram.setupComplete();   computeProfileStats(); } }
            },
            State {
                name: "SaveWorkout"
                PropertyChanges { target: setupParamsLoader;    visible: false }
                PropertyChanges { target: stepButtons;          visible: false }
                PropertyChanges { target: step1Button;          selected: false }
                PropertyChanges { target: step2Button;          selected: false }
                PropertyChanges { target: step3Button;          selected: true }
                PropertyChanges { target: step2;                visible: false }
                PropertyChanges { target: step3;                visible: false }
                PropertyChanges { target: nextButton;           visible: false }
                PropertyChanges { target: startButton;          visible: false }
                PropertyChanges { target: saveButton;           visible: false }
                PropertyChanges { target: saveProgLoader;       visible: true }
            }
        ]
    }

    // Title Text
    Text {
        id: prog_setup_program_name
        anchors.left: invisibleBackArea.right
        anchors.leftMargin: 30 * main.hScaleFactor
        anchors.top: parent.top
        anchors.topMargin: 43 * main.vScaleFactor
        anchors.right: parent.right
        anchors.rightMargin: 313 * main.hScaleFactor
        text: programNameTranslated
        visible: (setupScreenManager.state != "Step3")
        color: main.colorFontLight
        font.pixelSize: 20 * main.vScaleFactor
        font.bold: false
    }

    // Custom Workout Tab buttons
    RowLayout {
        id: stepButtons
        anchors {
            left: parent.left
            leftMargin: 1100 * main.hScaleFactor
            top: parent.top
            topMargin: 30 * main.vScaleFactor
            right: parent.right
            rightMargin: 100 * main.hScaleFactor
            bottom: parent.bottom
            bottomMargin: 685 * main.vScaleFactor
        }
        spacing: 5 * main.hScaleFactor

        StandardButton {
            id: step1Button
            selected: true
            buttonText: "1"
            width: 60 * main.hScaleFactor
            height: 40 * main.vScaleFactor
            label.font.pixelSize: 0.8 * height
            label.anchors.bottomMargin: 0
            label.anchors.verticalCenter: step1Button.verticalCenter
            onClicked: {
                setupScreenManager.state = "Step1"
                selected = true
                step2Button.selected = false
                step3Button.selected = false
            }
            isNumberLabel: true
        }

        StandardButton {
            id: step2Button
            buttonText: "2"
            width: 60 * main.hScaleFactor
            height: 40 * main.vScaleFactor
            label.font.pixelSize: 0.8 * height
            label.anchors.bottomMargin: 0
            label.anchors.verticalCenter: step2Button.verticalCenter
            onClicked: {
                basicSetup.validateEntries()
                setupScreenManager.state = "Step2"
                selected = true
                step1Button.selected = false
                step3Button.selected = false
            }
            isNumberLabel: true
        }

        StandardButton {
            id: step3Button
            buttonText: "3"
            width: 60 * main.hScaleFactor
            height: 40 * main.vScaleFactor
            label.font.pixelSize: 0.8 * height
            label.anchors.bottomMargin: 0
            label.anchors.verticalCenter: step3Button.verticalCenter
            onClicked: {
                basicSetup.validateEntries()
                setupScreenManager.state = "Step3"
                selected = true
                step1Button.selected = false
                step2Button.selected = false
            }
            isNumberLabel: true
        }
    }

    Column {
        id: actionButtons

        spacing: 10 * main.hScaleFactor
        anchors {
            right: parent.right
            rightMargin: 90 * main.hScaleFactor
            bottom: parent.bottom
            bottomMargin: 25 * main.vScaleFactor
        }
        width: buttonWidth

        property int buttonWidth: 138 * main.hScaleFactor

        // Bluetooth Heart Rate Device Image For Popup
        StandardButton {
            id: bluetoothHeartrateButton

            width: parent.buttonWidth
            height: 100 * main.vScaleFactor
            visible: targetHRType != targetHRTypeEnum.invalid && setupScreenManager.state !== "SaveWorkout"
            buttonText: qsTr("Heart Rate Monitor") + application.emptyString
            onClicked: {
                main.showMediaBrowser = true
                ipod_window.dialogState = "hr"
            }

            label.anchors.verticalCenterOffset: 0.23 * bluetoothHeartrateButton.height
            icon.anchors.topMargin: 8 * main.vScaleFactor
            icon.anchors.bottomMargin: 50 * main.vScaleFactor

            icon.sourceName: "Bluetooth/Bluetooth_HRicon"
        }

        // Save button (Save a custom workout)
        StandardButton {
            id: saveButton

            width: parent.buttonWidth
            height: 78 * main.vScaleFactor
            icon.anchors.topMargin: 5 * main.vScaleFactor
            icon.anchors.bottomMargin: 30 * main.vScaleFactor
            icon.sourceName: "WF/WF_Button_Saved_Workouts"
            buttonText: qsTr("SAVE?") + application.emptyString
            textColor: main.colorFontLight
            onClicked: {
                mainProgItem.previousState = setupScreenManager.state
                setupScreenManager.state = "SaveWorkout"
            }
        }

        // Start button (begins workout)
        StandardButton {
            id: startButton

            width: parent.buttonWidth
            height: 92 * main.vScaleFactor
            enabled: !main.eGymCardioTest || eGym.fitnessLevel !== ""
            sourceName: "HomeScreen/QuickStartIcon"
            onClicked: {
                console.debug("START!!!!")
                if(main.eGymCardioTest)
                    eGym.startTest()
                else
                    mainProgItem.doStart()
            }
        }

        // Next button (Goes between the custom workout setup tabs)
        StandardButton {
            id: nextButton

            width: parent.buttonWidth
            height: 92 * main.vScaleFactor
            sourceName: "HomeScreen/QuickStartIcon"
            onClicked: mainProgItem.nextState()
        }
    }

    // Custom Speed workout only
    ConfirmationPrompt {
        id: customSpeedWarningConfirmation
        blockKeys: true
        x: 300 * main.hScaleFactor
        y: 300 * main.vScaleFactor
        visible: false
        prompt: qsTr("Start workout?") + application.emptyString
        yesText: qsTr("ACCEPT") + application.emptyString
        noText: qsTr("RETURN TO SETUP") + application.emptyString
        onYesClicked: {
            mainProgItem.startProgram()
            hide()
        }
        onNoClicked: {
            setupScreenManager.state = "Step2"
            hide()
        }
        onTimedOut: hide()
    }

    // Gerkins & WFI Test only
    ConfirmationPrompt {
        id: gerkinTestPrompt
        blockKeys: true
        textSize: 20 * main.vScaleFactor
        x: 300 * main.hScaleFactor
        y: 200 * main.vScaleFactor
        visible: false
        prompt: {
            var msg = (workoutProgram.programIndex === WorkoutProgram.STEP_WFI_TEST_PROG) ? qsTr("Warning - The WFI Fitness Test can reach high levels of speed and heart rate.")
                      : qsTr("Warning - The Gerkin Fitness Test can reach high levels of speed, incline, and heart rate.")
            msg += " " + qsTr("Press ACCEPT if you wish to continue.")

            return msg
        }
        yesText: qsTr("ACCEPT") + application.emptyString
        noText: qsTr("RETURN TO SETUP") + application.emptyString
        onYesClicked: {
            mainProgItem.startProgram()
            hide()
        }
        onNoClicked: hide()
        onTimedOut: hide()
    }

    onShowHRChart: hrGuide.show()

    Loader {
        id: hrGuide

        Connections {
            target: hrGuide.item
            onClose: hrGuide.hide()
            ignoreUnknownSignals: true
        }

        function show() {
            source = "HeartRateGuide.qml";
        }

        function hide() {
            source = "";
        }

        anchors.fill: parent
        z: 99
    }
}
