import QtQuick 2.12
import WorkoutManager 1.0
import Components 1.0
import QtQuick.Layouts 1.12
import "../string_format.js" as Format
import "program_setup.js" as Fields
import "setup_params.js" as SetupJS
import ".."

Item {
    id: root

    property alias model: myListView.model
    property QtObject timeItem
    property QtObject distItem
    property QtObject calItem
    property bool timeGoal: true
    property bool distGoal: false
    property int maxCalPerHr: 1600
    property int paramTextSize: 18 * main.vScaleFactor
    property int numTextSize: 51 * main.vScaleFactor
    property int unitsTextSize: 12 * main.vScaleFactor
    property int selectedIndex: 0
    property int startIndex: 0
    property bool valueEntered: false

    property bool isElevationType: false
    property bool isAdjustWorkout: false
    property bool isCruiseControlEnabled: workoutManager.cruiseControlEnabled

    onModelChanged: {
        updateModel()
        console.debug("model updated")
    }

    function setParameter(type, value) {
        return SetupJS.setParameter(type, value);
    }

    function validateEntries() {
        keypad.submit(); // submit current item
        return SetupJS.validateEntries();
    }

    function recalcMaxSegments() {
        calculateMaxSegmentsFromTime();
    }

    function calculateMaxSegmentsFromTime() {
        if (!myListView.model)
            return

        var timeMins = SetupJS.getParameter(WorkoutProgram.TIME_TYPE)
        var maxSegs = Math.max(2, Math.min(30, timeMins))
        var item = root.model.find("fieldType", WorkoutProgram.SEGMENT_TYPE)
        if(!item)   // no segment count setup field, nothing to do
            return
        var index = item.fieldIndex - 1
        root.model.setProperty(index, "maxValue", maxSegs)
        if (myListView.contentItem !== undefined && (SetupJS.getParameter(WorkoutProgram.SEGMENT_TYPE) > maxSegs || myListView.contentItem.children[index + 1]._slider3.value < maxSegs)) {
            // the next line is needed because the slider's binding to model.dataValue gets broken somewhere
            myListView.contentItem.children[index + 1]._slider3.value = maxSegs
        }
    }

    function updateModel() {
        if (root.model) {
            SetupJS.modelPostProcessing()
        }
    }

    function updateBaseModel() {
        updateModel()
        workoutProgram.collectDataFromAdjustSetup(workoutManager.currentSegment)
        workoutManager.adjustWorkout()
    }

    onSelectedIndexChanged: {
        keypad.clear()
        var item = model.get(root.selectedIndex)

        if(!item) {
            return
        }

        if(item.units === "%") {
            keypad.length = 3
            keypad.decimalPlaces = 1
        } else {
            keypad.length = SetupJS.getMaxDigit(item.maxValue) + item.precision
            keypad.decimalPlaces = item.precision
        }

        keypad.units = item.units
    }

    // Validate data relationships when any value is changed
    Connections {
        target: root.model ? root.model : null
        ignoreUnknownSignals: true
        onDataChanged: {
            if((isTreadmill || main.isStepmill) && calItem) { // Treadmill calorie goal setup
                SetupJS.validateTreadCalorieGoal()
            }

            if (!isTreadmill && !main.isStepmill && calItem) {
                SetupJS.validateNonTreadCalorieGoal()
            }
        }
    }

    Connections {
        target: timeItem
        onDataChanged: {
            if(timeItem.dataValue > 0) {
                root.timeGoal = true
                root.distGoal = false
            }
        }
    }

    Connections {
        target: distItem
        onDataChanged: {
            if(distItem.dataValue > 0) {
                root.timeGoal = false
                root.distGoal = true
            }
        }
    }

    Rectangle {
        id: calWarning
        visible: false
        anchors {
            left: parent.left
            leftMargin: 42 * main.hScaleFactor
            bottom: parent.bottom
            bottomMargin: 10 * main.vScaleFactor
        }
        width: 354 * main.hScaleFactor
        height: 150 * main.vScaleFactor

        color: main.colorDialogBackground

        ScalableImage {
            id: calCautionSymbol
            anchors.verticalCenter: parent.verticalCenter
            sourceName: "Icon_Warning"
        }

        Text {
            id: calCautionTitle
            text: qsTr("CAUTION") + application.emptyString
            color: "orange"
            font.bold: false
            font.pixelSize: 18 * main.vScaleFactor
            anchors {
                left: calCautionSymbol.right
                leftMargin: 5 * main.hScaleFactor
                top: parent.top
                topMargin: 10 * main.vScaleFactor
            }
        }

        Text {
            text: qsTr( "The calorie goal chosen is too high for selected workout time.") + application.emptyString
            wrapMode: Text.WordWrap
            font.pixelSize: 16 * main.vScaleFactor
            color: main.colorFontLight
            anchors {
                left: calCautionSymbol.right
                leftMargin: 5 * main.hScaleFactor
                top: calCautionTitle.bottom
                topMargin: 5 * main.vScaleFactor
                right: parent.right
                rightMargin: 5 * main.hScaleFactor
            }
        }
    }

    Item {
        id: contentArea
        width: root.width - (root.isElevationType || root.isAdjustWorkout ? 0 : 407) * main.hScaleFactor
        height: root.height
        anchors.horizontalCenter: parent.horizontalCenter
        anchors.top: parent.top

        Text {
            id: prog_setup_desc_text

            property real defaultPixelSize: 14 * main.vScaleFactor

            visible: !isAdjustWorkout
            anchors {
                left: parent.left
                leftMargin: 120 * main.hScaleFactor
                top: parent.top
                topMargin: 85 * main.vScaleFactor
                right: parent.right
                rightMargin: 313 * main.hScaleFactor
            }
            height: 80 * main.vScaleFactor
            text: {
                if (root.model && workoutProgram.programIndex >= WorkoutProgram.MANUAL_PROG && workoutProgram.programIndex < WorkoutProgram.SAVED_WORKOUT) {
                    // Subtract 3 from programIndex to account for NO_PROG and QUICK_START_PROG, TREAD_QUICK_START_PROG, and STEP_QUICK_START_PROG
                    var str = Fields.workout_programs()[(workoutProgram.programIndex - 4)][(root.model.get(root.selectedIndex).fieldIndex- 1)]
                    var trans = str ? (application.translateText("Setup", str) + application.emptyString) : ""

                    console.debug("workout Program: "+(workoutProgram.programIndex - 3)+":"+(root.model.get(root.selectedIndex).fieldIndex- 1))

                    // CAN-746 Workout setup time text doesn't match max. workout time setting
                    if (str === Fields.prog_setup_time_99_desc || str === Fields.prog_setup_manual_time_desc || str === Fields.prog_setup_time_99_manual_desc || str === Fields.prog_setup_time_60_desc) {
                        trans = trans.replace("99", options.workoutTimeMax)
                    }

                    return trans
                } else {
                    return "NA"
                }
            }
            color: main.colorFontLight
            wrapMode: Text.WordWrap
            font.pixelSize: defaultPixelSize
            font.bold: false
        }

        Keypad {
            id: keypad

            property variant model: myListView.model
            property int currentFieldType: getFieldType()

            function getFieldType() {
                if(!model) {
                    return -1;
                }

                var field = model.get(root.selectedIndex).fieldType;

                if(!field) {
                    return -1;
                }

                return field;
            }

            function keypadAllowed(fieldType){

                //console.debug("keypadcheck: "+ fieldType)
                switch (fieldType) {
                case WorkoutProgram.ELEVATION_TYPE:
                case WorkoutProgram.GENDER_TYPE:
                case WorkoutProgram.FIXED_DISTANCE_TYPE:
                case WorkoutProgram.DISTANCE_QUICK_TYPE:
                case WorkoutProgram.FLOOR_TYPE:
                    return false
                default:
                    return true
                }
            }

            z: root.z + 2
            color: "transparent"
            anchors {
                top: parent.top
                topMargin: isAdjustWorkout ? 28 * main.vScaleFactor : 148 * main.vScaleFactor
                right: parent.right
                rightMargin: (isAdjustWorkout ? 20 : root.isElevationType ? 240 : 30 )* main.hScaleFactor
            }
            show_entry: false
            allow_close: false

            keyWidth: 70 * main.hScaleFactor
            keyHeight: 73 * main.vScaleFactor
            visible: {
               return keypadAllowed( getFieldType() )
            }
            enabled: !(currentFieldType == WorkoutProgram.TARGET_HEARTRATEREADONLY_TYPE || currentFieldType == WorkoutProgram.TIMEREADONLY_TYPE)  // don't allow changing read-only field

            onValueEntered: {
                if(keypad.value && keypad.value != "0.0") {
                    var item = model.get(root.selectedIndex)
                    var tmp
                    root.valueEntered = true

                    if(item.units === "%") {
                        tmp = workoutProgram.tableIndexOfClosestValue(workoutProgram.inclineTable, keypad.value * 10)
                        model.setProperty(root.selectedIndex, "dataValue", tmp)
                    } else if (SetupJS.isSpeedType(item.fieldType) && options.metricUnits) {
                        // when the user is using metric, find the closest possible mph value in the speed table to correct their entry
                        tmp = Globals.kmToMile(parseFloat(value))
                        tmp = workoutProgram.tableClosestValue(workoutProgram.speedTable, Math.round(tmp * 10))

                        if (isNaN(tmp)) {
                            tmp = workoutProgram.speedTable[0]
                        }

                        model.setProperty(root.selectedIndex, "dataValue", tmp / 10)
                    } else if (SetupJS.isStrideType(item.fieldType) && options.metricUnits) {
                        tmp = Globals.cmToInch(parseFloat(value));
                        tmp = Math.round(tmp);
                        model.setProperty(root.selectedIndex, "dataValue", tmp);
                    } else if (currentFieldType !== WorkoutProgram.ELEVATION_TYPE) {
                        model.setProperty(root.selectedIndex, "dataValue", keypad.value);
                    }

                    root.valueEntered = false
                    var corrections = SetupJS.validateParam(root.selectedIndex)

                    if (item.fieldType === WorkoutProgram.SEGMENT_TYPE) {
                        mainProgItem.numSegs = model.get(root.selectedIndex).dataValue
                    }

                    root.valueEntered = false

                    // go to next field when enter pressed, if entry was not corrected by validation
                    if(corrections === 0) {
                        root.selectedIndex++
                    }
                }
            }
            onKeyPress: {
                if (key != ">") {
                    var item = model.get(root.selectedIndex);
                    if (item !== undefined && keypad.value) {
                        var val = Number(keypad.value);

                        if (SetupJS.isSpeedType(item.fieldType) && options.metricUnits) {
                            val = Globals.kmToMile(parseFloat(val))
                        } else if (SetupJS.isStrideType(item.fieldType) && options.metricUnits) {
                            val = Globals.cmToInch(parseFloat(val))
                            val = Math.round(val)
                        } else if (item.fieldType === WorkoutProgram.INCLINE_TYPE || item.fieldType === WorkoutProgram.MAX_INCLINE_TYPE) {
                            val = workoutProgram.tableIndexOfClosestValue(workoutProgram.inclineTable, parseFloat(val) * 10)
                        }

                        if(item.fieldType == WorkoutProgram.ELEVATION_TYPE)
                            model.setProperty(root.selectedIndex, "dataValue", Qt.point(val, val));
                        else
                            model.setProperty(root.selectedIndex, "dataValue", val);

                        if (item.fieldType === WorkoutProgram.TIME_TYPE) {
                            calculateMaxSegmentsFromTime()
                        }
                    }
                }
            }
        }

        Flickable {
            flickableDirection: Flickable.VerticalFlick
            contentHeight: contentItem.childrenRect.height
            boundsBehavior: Flickable.StopAtBounds
            clip: true

            anchors {
                left: parent.left
                leftMargin: isAdjustWorkout ? 10 * main.hScaleFactor :120 * main.hScaleFactor
                top: isAdjustWorkout ? parent.top : prog_setup_desc_text.bottom
                right: parent.right
                bottom: parent.bottom
                bottomMargin: 20 * main.vScaleFactor
            }

            Column {
                spacing: 20 * main.vScaleFactor
                width: parent.width

                Repeater {
                    id: myListView

                    clip: true
                    delegate: myDelegate

                    Component {
                        id: myDelegate

                        DualBox {
                            id: dataElement
                            expandTime: 500
                            visible: (!isAdjustWorkout && !isCruseHR ) || (isAdjustWorkout && fieldAdjustable && isCruiseControlEnabled == isCruseHR)
                            property real initialWidth: 470 * main.hScaleFactor
                            property real initialHeight: 71 * main.vScaleFactor
                            property bool isTransparentHRC: false
                            property bool isReadOnly: false
                            property bool isCI: index == root.selectedIndex
                            property bool isCruseHR: !workoutProgram.isHRCProgram && model.fieldType === WorkoutProgram.TARGET_HEARTRATE_TYPE

                            onIsCIChanged: {
                                if(isCI && isClickable) {
                                    expand()
                                    clickToExpand = false
                                } else {
                                    collapse()
                                    clickToExpand = true
                                }
                            }
                            onClicked: {
                                if(isTransparentHRC || isReadOnly) {
                                    console.debug("HRC trnas")
                                    return
                                }

                                root.selectedIndex = index;

                                SetupJS.validateParam(root.selectedIndex)
                                var item = root.model.get(root.selectedIndex)

                                var tmp
                                if(SetupJS.isSpeedType(item.fieldType)&& options.metricUnits) {
                                    tmp = (keypad.keyEntryText.length > 0) ? keypad.value : parseFloat(item.dataValue)
                                    tmp = workoutProgram.tableClosestValue(workoutProgram.speedTable, Math.round(tmp * 10))

                                    if(isNaN(tmp)) {
                                        tmp = workoutProgram.speedTable[0]
                                    }

                                    root.model.setProperty(root.selectedIndex, "dataValue", tmp / 10)
                                } else if(item.units === "%") {
                                    if(keypad.keyEntryText.length > 0) {
                                        tmp = parseFloat(keypad.value)
                                        tmp = workoutProgram.tableIndexOfClosestValue(workoutProgram.inclineTable,  Math.round(tmp * 10))
                                    } else {
                                        tmp = parseFloat(item.dataValue)
                                    }

                                    if(isNaN(tmp) || tmp < 0) {
                                        tmp = 0
                                    }

                                    root.model.setProperty(root.selectedIndex, "dataValue", tmp)
                                }

                                buzzer.buttonPress()
                            }

                            Component.onCompleted: populateElement(model.fieldType)

                            function populateElement(fieldtype) {
                                switch(fieldtype) {
                                case WorkoutProgram.MAX_SPEED_TYPE:
                                case WorkoutProgram.MAX_INCLINE_TYPE:
                                case WorkoutProgram.SPEED_TYPE:
                                case WorkoutProgram.RUN_SPEED_TYPE:
                                case WorkoutProgram.WORK_SPEED_TYPE:
                                case WorkoutProgram.WALK_SPEED_TYPE:
                                case WorkoutProgram.REST_SPEED_TYPE:
                                case WorkoutProgram.PACE_TYPE:
                                case WorkoutProgram.WORKLOAD_TYPE:
                                case WorkoutProgram.AGE_TYPE:
                                case WorkoutProgram.WEIGHT_TYPE:
                                case WorkoutProgram.HEIGHT_TYPE:
                                case WorkoutProgram.DISTANCE_TYPE:
                                case WorkoutProgram.INCLINE_TYPE:
                                case WorkoutProgram.CALORIES_TYPE:
                                case WorkoutProgram.MIN_STRIDE_TYPE:
                                case WorkoutProgram.MAX_STRIDE_TYPE:
                                    collapsedLoader.sourceComponent = topcomp;
                                    extensionLoader.sourceComponent = bottomcomp_slider;
                                    break;
                                case WorkoutProgram.TIME_TYPE:
                                case WorkoutProgram.WORKTIME_TYPE:
                                case WorkoutProgram.RESTTIME_TYPE:
                                    collapsedLoader.sourceComponent = topcomp_time;
                                    extensionLoader.sourceComponent = bottomcomp_slider;
                                    break;
                                case WorkoutProgram.ELEVATION_TYPE:
                                    collapsedLoader.sourceComponent = topcomp_dualslider;
                                    if(isAdjustWorkout) {
                                        extensionLoader.sourceComponent = bottomcomp_slider;
                                    }
                                    else
                                        extensionLoader.sourceComponent = bottomcomp_dualslider;
                                    break;
                                case WorkoutProgram.LEVEL_TYPE:
                                    collapsedLoader.sourceComponent = (main.isTreadmill) ? topcomp_level : topcomp;
                                    extensionLoader.sourceComponent = bottomcomp_slider;
                                    break;
                                case WorkoutProgram.GENDER_TYPE:
                                    collapsedLoader.sourceComponent = topcomp;
                                    extensionLoader.sourceComponent = bottomComp_gender;
                                    break;
                                case WorkoutProgram.DISTANCE_QUICK_TYPE:
                                    collapsedLoader.sourceComponent = topcomp;
                                    extensionLoader.sourceComponent = bottomcomp_distquick;
                                    break;
                                case WorkoutProgram.FLOOR_TYPE:
                                    collapsedLoader.sourceComponent = topcomp;
                                    extensionLoader.sourceComponent = bottomcomp_floor_stepmill
                                    break;
                                case WorkoutProgram.WORK_HEARTRATE_TYPE:
                                case WorkoutProgram.REST_HEARTRATE_TYPE:
                                case WorkoutProgram.TARGET_HEARTRATE_TYPE:
                                    collapsedLoader.sourceComponent = topComp_HRDetail;
                                    extensionLoader.sourceComponent = bottomcomp_slider;
                                    break;
                                case WorkoutProgram.TARGET_HEARTRATEREADONLY_TYPE:
                                    collapsedLoader.sourceComponent = topComp_HRDetail;
                                    extensionLoader.sourceComponent = undefined;
                                    dataElement.isTransparentHRC = true;
                                    dataElement.isReadOnly = true;
                                    dataElement.isClickable = false;
                                    break;
                                case WorkoutProgram.CALORIE_POTENTIAL_TYPE:
                                case WorkoutProgram.FIXED_DISTANCE_TYPE:
                                    collapsedLoader.sourceComponent = topcomp;
                                    extensionLoader.sourceComponent = undefined;
                                    dataElement.isReadOnly = true;
                                    dataElement.isClickable = false;
                                    break;
                                case WorkoutProgram.TIMEREADONLY_TYPE:
                                    collapsedLoader.sourceComponent = topcomp_time;
                                    extensionLoader.sourceComponent = undefined;
                                    dataElement.isReadOnly = true;
                                    dataElement.isClickable = false;
                                    break;
                                case WorkoutProgram.SEGMENT_TYPE:
                                    collapsedLoader.sourceComponent = topcomp;
                                    extensionLoader.sourceComponent = bottomcomp_segment;
                                    break;
                                }
                            }

                            Component { // Top Component - Basic
                                id: topcomp

                                Item {
                                    width: dataElement.initialWidth
                                    height: dataElement.initialHeight

                                    Item {
                                        anchors.fill: parent
                                        anchors.margins: 5*main.scaleFactor

                                        TextLabel {
                                            id: itemName
                                            text: qsTranslate("WorkoutSetup", SetupJS.getItemLabel(model)) + SetupJS.getItemAdditionalLabel(model) + application.emptyString
                                            font.pixelSize: root.paramTextSize
                                            anchors.left: parent.left
                                            anchors.leftMargin: 10 * main.hScaleFactor
                                            anchors.verticalCenter: parent.verticalCenter
                                            width: 0.6 * dataElement.initialWidth
                                            horizontalAlignment: main.rightToLeft ? Text.AlignRight : Text.AlignLeft
                                        }

                                        NumberLabel {
                                            id: itemValue
                                            text: (isCI && keypad.keyEntryText.length > 0) ? keypad.value : SetupJS.getValueString(model)
                                            font.pixelSize: root.numTextSize
                                            anchors.right: itemUnits.left
                                            anchors.rightMargin: 20 * main.hScaleFactor
                                            anchors.verticalCenter: parent.verticalCenter
                                        }

                                        Text {
                                            id: itemUnits
                                            text: model.units
                                            font.pixelSize: root.unitsTextSize
                                            font.bold: false
                                            anchors.right: parent.right
                                            anchors.rightMargin: 10 * main.hScaleFactor
                                            anchors.bottom: itemName.bottom
                                            color: main.colorFontLight
                                            wrapMode: Text.WordWrap
                                        }
                                    }
                                }
                            }

                            Component { // Top Component - Level + Max Incline Message
                                id: topcomp_level

                                Item {
                                    width: dataElement.initialWidth
                                    height: dataElement.initialHeight

                                    Item {
                                        anchors.fill: parent
                                        anchors.margins: 5*main.scaleFactor
                                        anchors.leftMargin: 10 * main.hScaleFactor

                                        TextLabel {
                                            id: itemName
                                            text: qsTranslate("WorkoutSetup", SetupJS.getItemLabel(model)) + SetupJS.getItemAdditionalLabel(model) + application.emptyString
                                            horizontalAlignment: main.rightToLeft ? Text.AlignRight : Text.AlignLeft
                                            font.pixelSize: root.paramTextSize
                                            anchors.left: parent.left
                                            anchors.top: parent.top
                                            anchors.topMargin: 6 * main.vScaleFactor
                                            width: 0.6* dataElement.initialWidth
                                        }

                                        Row {
                                            id: levelDetails

                                            anchors.top: itemName.bottom
                                            x: 10 * main.hScaleFactor
                                            width: 0.6 * dataElement.initialWidth
                                            LayoutMirroring.enabled: main.rightToLeft
                                            LayoutMirroring.childrenInherit: false

                                            Text {
                                                id: levelSubLabel
                                                anchors.bottom: parent.bottom
                                                text: SetupJS.levelDetailsGetLabel() + ":" + application.emptyString
                                                verticalAlignment: Text.AlignBottom
                                                horizontalAlignment: main.rightToLeft ? Text.AlignRight : Text.AlignLeft
                                                font.pixelSize: 10 * main.vScaleFactor
                                                font.bold: false
                                                smooth: true
                                                color: main.colorFontLight
                                                z: 2
                                            }

                                            Item {
                                                width: 5 * main.hScaleFactor
                                                height: levelSubValue.height
                                            }

                                            Row {
                                                anchors.bottom: parent.bottom
                                                NumberLabel {
                                                    id: levelSubValue
                                                    anchors.bottom: parent.bottom
                                                    text: SetupJS.levelDetailsGetSubValue(model.dataValue)
                                                    verticalAlignment: Text.AlignBottom
                                                    font.pixelSize: 14 * main.vScaleFactor
                                                    z: 2
                                                }

                                                Text {
                                                    id: levelSubUnits
                                                    anchors.bottom: parent.bottom
                                                    text: SetupJS.levelDetailsGetUnits() + application.emptyString
                                                    verticalAlignment: Text.AlignBottom
                                                    horizontalAlignment: Text.AlignLeft
                                                    font.pixelSize: 8 * main.vScaleFactor
                                                    font.bold: false
                                                    smooth: true
                                                    color: main.colorFontLight
                                                    z: 2
                                                }
                                            }
                                        }


                                        NumberLabel {
                                            id: itemValue
                                            text: (isCI && keypad.keyEntryText.length > 0) ? keypad.value : SetupJS.getValueString(model)
                                            font.pixelSize: root.numTextSize
                                            anchors.right: itemUnits.left
                                            anchors.rightMargin: 20 * main.hScaleFactor
                                            anchors.verticalCenter: parent.verticalCenter
                                        }

                                        Text {
                                            id: itemUnits
                                            text: model.units
                                            font.pixelSize: root.unitsTextSize
                                            font.bold: false
                                            anchors.right: parent.right
                                            anchors.rightMargin: 10 * main.hScaleFactor
                                            anchors.bottom: itemValue.bottom
                                            color: main.colorFontLight
                                            wrapMode: Text.WordWrap
                                        }
                                    }
                                }
                            }

                            Component { // Top Component - Time
                                id: topcomp_time

                                Item {
                                    width: dataElement.initialWidth
                                    height: dataElement.initialHeight

                                    Item {
                                        anchors.fill: parent
                                        anchors.margins: 5*main.scaleFactor

                                        TextLabel {
                                            id: itemName
                                            text: qsTranslate("WorkoutSetup", SetupJS.getItemLabel(model)) + SetupJS.getItemAdditionalLabel(model) + application.emptyString
                                            horizontalAlignment: main.rightToLeft ? Text.AlignRight : Text.AlignLeft
                                            width: 0.6 * dataElement.initialWidth
                                            font.pixelSize: root.paramTextSize
                                            anchors.left: parent.left
                                            anchors.leftMargin: 10 * main.hScaleFactor
                                            anchors.verticalCenter: parent.verticalCenter
                                        }

                                        NumberLabel {
                                            id: itemValue
                                            text: Format.formatTimeString((isCI && keypad.keyEntryText.length > 0) ? keypad.value*60 : SetupJS.getValueString(model)*60)
                                            font.pixelSize: root.numTextSize
                                            anchors.right: parent.right
                                            anchors.rightMargin: 10 * main.hScaleFactor
                                            anchors.verticalCenter: parent.verticalCenter
                                        }
                                    }
                                }
                            }

                            Component { // Top Component - Elevation Dual Slider
                                id: topcomp_dualslider

                                Item {
                                    width: dataElement.initialWidth

                                    height: dataElement.initialHeight

                                    Item {
                                        anchors.fill: parent
                                        anchors.margins: 5*main.scaleFactor

                                        TextLabel {
                                            id: itemName
                                            text: qsTranslate("WorkoutSetup", SetupJS.getItemLabel(model)) + SetupJS.getItemAdditionalLabel(model) + application.emptyString

                                            width: 0.6 * dataElement.initialWidth
                                            horizontalAlignment: main.rightToLeft ? Text.AlignRight : Text.AlignLeft
                                            font.pixelSize: root.paramTextSize
                                            font.bold: false
                                            anchors.left: parent.left
                                            anchors.leftMargin: 10
                                            anchors.verticalCenter: parent.verticalCenter

                                            color: main.colorFontLight
                                        }

                                        NumberLabel {
                                            id: itemValue
                                            text: SetupJS.getValueString(model)
                                            font.pixelSize: root.numTextSize
                                            anchors.right: itemUnits.left
                                            anchors.rightMargin: 10
                                            anchors.verticalCenter: parent.verticalCenter
                                        }

                                        Text {
                                            id: itemUnits
                                            text: model.units
                                            font.pixelSize: root.unitsTextSize
                                            font.bold: false
                                            anchors.right: parent.right
                                            anchors.rightMargin: 10
                                            anchors.bottom: itemName.bottom
                                            color: main.colorFontLight
                                            wrapMode: Text.WordWrap
                                        }
                                    } // RowLayout
                                } // Item
                            }

                            Component { // Top component - Heart Rate
                                id: topComp_HRDetail

                                Item {
                                    width: dataElement.initialWidth
                                    height: dataElement.initialHeight * 1.5

                                    Item {
                                        anchors.top: parent.top
                                        anchors.bottom: parent.bottom
                                        anchors.left: parent.left
                                        anchors.margins: 5*main.scaleFactor
                                        width: parent.width - itemGridLayout.width

                                        TextLabel {
                                            id: itemName

                                            horizontalAlignment: main.rightToLeft ? Text.AlignRight : Text.AlignLeft
                                            anchors.top: parent.top
                                            anchors.topMargin: 15* main.vScaleFactor
                                            anchors.left: parent.left
                                            anchors.leftMargin: 10 * main.hScaleFactor
                                            width: 0.9 * parent.width
                                            wrapMode: Text.Wrap

                                            text: qsTranslate("WorkoutSetup", SetupJS.getItemLabel(model)) + SetupJS.getItemAdditionalLabel(model) + application.emptyString
                                            font.pixelSize: root.paramTextSize
                                            font.bold: false
                                            color: main.colorFontLight
                                        }

                                        MouseArea {
                                            id: hrChartButton
                                            width: 30 * main.hScaleFactor
                                            height: 30 * main.vScaleFactor
                                            anchors.left: itemName.left
                                            anchors.top: itemName.bottom
                                            anchors.topMargin: 5 * main.vScaleFactor

                                            onClicked: showHRChart();
                                            onReleased: {
                                                buzzer.buttonPress()
                                            }

                                            Rectangle {
                                                anchors.fill: parent
                                                color: main.colorGenericStimulusArea
                                                opacity: 0.3
                                                visible: main.showStimulusArea
                                            }

                                            Rectangle {
                                                anchors.fill: parent
                                                color: "transparent"
                                                visible: dataElement.expanded
                                                border.color: main.colorUIBorder
                                            }

                                            ScalableImage {
                                                anchors.fill: parent
                                                anchors.margins: 5 * main.scaleFactor
                                                sourceName: "WF/WF_Button_HRC"
                                                fillMode: Image.PreserveAspectFit
                                            }
                                        }

                                        TextLabel {
                                            id: hrcChartLabel
                                            text: qsTr("HRC CHART") + application.emptyString
                                            font.pixelSize: 12 * main.vScaleFactor
                                            font.bold: false
                                            color: main.colorFontLight
                                            width: 60 * main.hScaleFactor
                                            height: 30 * main.vScaleFactor
                                            anchors.verticalCenter: hrChartButton.verticalCenter
                                            //                                                anchors.top: itemName.bottom
                                            anchors.left: hrChartButton.right
                                            anchors.leftMargin: 10 * main.hScaleFactor
                                            verticalAlignment: Text.AlignVCenter
                                        }
                                    }

                                    Grid {
                                        id: itemGridLayout

                                        anchors.top: parent.top
                                        anchors.bottom: parent.bottom
                                        anchors.right: parent.right
                                        anchors.margins: 10*main.scaleFactor

                                        columns: 2
                                        rows: 2
                                        columnSpacing: 10 * main.hScaleFactor

                                        horizontalItemAlignment: Grid.AlignRight
                                        verticalItemAlignment: Grid.AlignVCenter

                                        NumberLabel { // Top left
                                            id: itemValue
                                            text: SetupJS.getValueString(model)
                                            font.pixelSize: 51 * main.vScaleFactor
                                            horizontalAlignment: Text.AlignRight
                                            wrapMode: Text.NoWrap
                                        }

                                        Text { // Top right
                                            id: itemUnits
                                            text: model.units
                                            font.pixelSize: root.unitsTextSize
                                            font.bold: false
                                            color: main.colorFontLight
                                            wrapMode: Text.NoWrap
                                        }

                                        NumberLabel { // Bottom left
                                            id: hrPercentageValue
                                            text: model.heartRatePercentage
                                            font.pixelSize: 28 * main.vScaleFactor
                                            horizontalAlignment: Text.AlignRight
                                            wrapMode: Text.NoWrap
                                        }

                                        Text { // Bottom right
                                            id: hrPercentageUnits
                                            text: qsTr("% MAX") + application.emptyString
                                            font.pixelSize: root.unitsTextSize
                                            font.bold: false
                                            color: main.colorFontLight
                                            wrapMode: Text.NoWrap
                                        }
                                    }
                                }
                            }

                            Component { // Bottom component - Slider
                                id: bottomcomp_slider

                                Item { // slider control box seen after expanding most simple items (age, weight, time, etc.)
                                    id: details

                                    height: 75 * main.vScaleFactor
                                    width: initialWidth
                                    anchors {
                                        left: parent.left
                                        leftMargin: 10 * main.hScaleFactor
                                        top: parent.top
                                        topMargin: 10 * main.vScaleFactor
                                    }

                                    property int value: sliderObject.value

                                    SliderControl {
                                        id: sliderObject
                                        repeat: true
                                        valueProxy: model.fieldType !== WorkoutProgram.ELEVATION_TYPE ? model.dataValue : model.dataValue.y
                                        Component.onCompleted: {
                                            if (model.fieldType === WorkoutProgram.ELEVATION_TYPE) {
                                                return // Avoid warning about assigning QPointF to double
                                            }

                                            if (model.dataValue !== undefined)
                                                value = model.dataValue
                                        }
                                        onValueChanged: {
                                            if (model.fieldType === WorkoutProgram.ELEVATION_TYPE) {
                                                if (!root.valueEntered && !valueProxyChange) {
                                                    var pt = Qt.point(model.dataValue.x, value);
                                                    root.model.setProperty(index, "dataValue", pt);
                                                    workoutProgram.setEndElevation(pt.y);
                                                    workoutManager.updateProjectedFinishTime();
                                                }
                                            }
                                            else {
                                                SetupJS.sliderObjectValueChanged(model, value, valueProxyChange)
                                                if (main.isAlpineRunner) { // Limit speed/incline ranges for Alpine Runner
                                                    if (model.fieldType === WorkoutProgram.SPEED_TYPE
                                                            || model.fieldType === WorkoutProgram.INCLINE_TYPE
                                                            || model.fieldType === WorkoutProgram.MAX_SPEED_TYPE
                                                            || model.fieldType === WorkoutProgram.MAX_INCLINE_TYPE) {
                                                        SetupJS.setAlpineCustomMaxValue(fieldType, value)
                                                    }
                                                }
                                            }
                                        }
                                        onInput: keypad.clear() // Clear the keypad whenever there is an input
                                        showValue: false
                                        maxValue: model.maxValue
                                        onMaxValueChanged: {
                                            if (model.fieldType === WorkoutProgram.SPEED_TYPE || model.fieldType === WorkoutProgram.INCLINE_TYPE) {
                                                sliderObject.valueChanged()
                                            }
                                        }
                                        minValue: model.fieldType !== WorkoutProgram.ELEVATION_TYPE ? model.minValue : model.dataValue.x
                                        value: model.fieldType !== WorkoutProgram.ELEVATION_TYPE ? model.dataValue : model.dataValue.y
                                        precision: model.precision
                                        stepValue: {
                                            if(model.fieldType === WorkoutProgram.WORKTIME_TYPE || model.fieldType === WorkoutProgram.RESTTIME_TYPE) {
//                                                console.log("Setting step value to 0.5 for Work/Rest Time Slider")
                                                return 0.5
                                            } else if (model.precision === 0) {
                                                return 1
                                            } else if (model.precision === 1) {
                                                return 0.1
                                            }
                                        }
                                        anchors.top: parent.top
                                        anchors.horizontalCenter: parent.horizontalCenter
                                        width: 350 * main.hScaleFactor
                                        height: 34 * main.vScaleFactor
                                    }
                                }
                            }

                            Component { // Bottom component - Dual Slider
                                id: bottomcomp_dualslider

                                Item { // slider control box seen after expanding elevation
                                    id: details

                                    height: 180 * main.vScaleFactor
                                    width: 0.7 * main.screenWidth
                                    anchors.left: parent.left
                                    anchors.top: parent.top

                                    DualSlider {
                                        id: elevationSlider

                                        property bool completed: false
                                        enabled: visible //&& elevationDetails.opacity > 0
                                        valueProxy: model.dataValue

                                        ListModel {
                                            id: mtEverestIndexPoints // NOTE: Left untranslated on purpose

                                            ListElement { value: 0; label: "SEA LEVEL"; offset: 3 }
                                            ListElement { value: 17000; label: "BASE CAMP"; offset: -10}
                                            ListElement { value: 19900; label: "LHO LA"; offset: 0}
                                            ListElement { value: 21300; label: "CAMP 2"; offset: 35}
                                            ListElement { value: 24500; label: "GENEVA\nSPUR" }
                                            ListElement { value: 26000; label: "CAMP 4"; offset: 20 }
                                            ListElement { value: 29029; label: "SUMMIT" }
                                        }

                                        ListModel {
                                            id: mtRainierIndexPoints
                                            ListElement { value: 0; label: "SEA LEVEL"; offset: 3}
                                            ListElement { value: 5440; label: "ALTA VISTA"; offset: 3}
                                            ListElement { value: 10188; label: "CAMP MUIR"; offset: 3}
                                            ListElement { value: 11600; label: "DISAPPOINTMENT\nCLEAVER"; offset: 20 }
                                            ListElement { value: 14411; label: "SUMMIT" }
                                        }

                                        ListModel {
                                            id: mtKilimanjaroIndexPoints

                                            ListElement { value: 0; label: "SEA LEVEL"; offset: 3 }
                                            ListElement { value: 5905; label: "MACHAME GATE" }
                                            ListElement { value: 13000; label: "SHIRA PEAK" }
                                            ListElement { value: 15174; label: "LAVA\nTOWER" }
                                            ListElement { value: 16893; label: "MAWENZI\nPEAK"; offset: 6}
                                            ListElement { value: 19341; label: "SUMMIT" }
                                        }

                                        function getIndexModelForWorkout() {
                                            switch (workoutProgram.programIndex) {
                                            case WorkoutProgram.MOUNT_EVEREST_PROG:
                                            case WorkoutProgram.STEP_MOUNT_EVEREST_PROG:
                                                return mtEverestIndexPoints;
                                            case WorkoutProgram.MOUNT_RAINIER_PROG:
                                            case WorkoutProgram.STEP_MOUNT_RAINIER_PROG:
                                                return mtRainierIndexPoints;
                                            case WorkoutProgram.MOUNT_KILIMANJARO_PROG:
                                            case WorkoutProgram.STEP_MOUNT_KILIMANJARO_PROG:
                                                return mtKilimanjaroIndexPoints;
                                            }

                                            return null;
                                        }

                                        Component.onCompleted: {
                                            if (model.dataValue && model.dataValue.x !== undefined) {
                                                elevationSlider.startValue = model.dataValue.x
                                                elevationSlider.finishValue = model.dataValue.y
                                                var pt = Qt.point(elevationSlider.startValue, elevationSlider.finishValue)
                                                myListView.model.setProperty(index, "dataValue", pt)
                                            }

                                            completed = true
                                        }

                                        function handleValueChange() {
                                            if (!completed) {
                                                return;
                                            }

                                            if (model.fieldType !== WorkoutProgram.ELEVATION_TYPE) {
                                                return;
                                            }

                                            var pt = Qt.point(elevationSlider.startValue, elevationSlider.finishValue);
                                            root.model.setProperty(index, "dataValue", pt);
                                        }

                                        onStartValueChanged: handleValueChange()
                                        onFinishValueChanged: handleValueChange()
                                        startDisplayValue: options.metricUnits ? Globals.formatWithSeparator(Globals.feetToMeters(startValue).toFixed(0)) : Globals.formatWithSeparator(startValue)
                                        finishDisplayValue: options.metricUnits ? Globals.formatWithSeparator(Globals.feetToMeters(finishValue).toFixed(0)) : Globals.formatWithSeparator(finishValue)
                                        showValue: false
                                        maxValue: model.maxValue
                                        minValue: model.minValue
                                        precision: model.precision
                                        trackWidth: 0.9 * parent.width
                                        stepValue: options.metricUnits ? 82.021 : 100.0 // 25 meters / 100 feet
                                        anchors.top: parent.top
                                        anchors.topMargin: parent.topMargin
                                        anchors.left: parent.left
                                        anchors.leftMargin: parent.leftMargin
                                        width: parent.width
                                        height: parent.height
                                        indexPoints: getIndexModelForWorkout()
                                    }
                                }
                            }

                            Component { // Bottom component - Distance Quick

                                id: bottomcomp_distquick

                                RowLayout {
                                    id: expandedDetails1
                                    height: 75 * main.vScaleFactor
                                    width: initialWidth

                                    StandardButton {
                                        id: fiveKButton

                                        property bool valueMatches: (dataValue == 5) && (units == "K")

                                        width: 94 * main.hScaleFactor
                                        height: 50 * main.vScaleFactor
                                        Layout.alignment: Qt.AlignVCenter | Qt.AlignHCenter
                                        Layout.bottomMargin: 10
                                        buttonText: qsTr("5K") + application.emptyString
                                        selected: valueMatches
                                        onClicked: {
                                            keypad.clear() // Clear the keypad whenever there is an input
                                            root.model.setProperty(index, "dataValue", 5)
                                            root.model.setProperty(index, "units", "K")
                                            mainProgItem.dataItemChanged()
                                        }
                                    }

                                    StandardButton {
                                        id: tenKButton

                                        property bool valueMatches: (dataValue == 10) && (units == "K")

                                        enabled: workoutProgram.programIndex !== WorkoutProgram.STRIDE_DISTANCE_PROG
                                        width: 94 * main.hScaleFactor
                                        height: 50 * main.vScaleFactor
                                        Layout.alignment: Qt.AlignVCenter | Qt.AlignHCenter
                                        Layout.bottomMargin: 10 * main.vScaleFactor
                                        buttonText: qsTr("10K") + application.emptyString
                                        selected: valueMatches
                                        onClicked: {
                                            keypad.clear() // Clear the keypad whenever there is an input
                                            root.model.setProperty(index, "dataValue", 10)
                                            root.model.setProperty(index, "units", "K")
                                            mainProgItem.dataItemChanged()
                                        }
                                    }

                                    StandardButton {
                                        id: twoMileButton

                                        property bool valueMatches: (dataValue == 2) && (units == "MI")

                                        enabled: workoutProgram.programIndex !== WorkoutProgram.STRIDE_DISTANCE_PROG
                                        width: 94 * main.hScaleFactor
                                        height: 50 * main.vScaleFactor
                                        Layout.alignment: Qt.AlignVCenter | Qt.AlignHCenter
                                        Layout.bottomMargin: 10 * main.vScaleFactor
                                        buttonText: qsTr("2MI") + application.emptyString
                                        selected: valueMatches
                                        onClicked: {
                                            keypad.clear() // Clear the keypad whenever there is an input
                                            root.model.setProperty(index, "dataValue", 2)
                                            root.model.setProperty(index, "units", "MI")
                                            mainProgItem.dataItemChanged()
                                        }
                                    }

                                    StandardButton {
                                        id: fourMileButton

                                        property bool valueMatches: (dataValue == 4) && (units == "MI")

                                        enabled: workoutProgram.programIndex !== WorkoutProgram.STRIDE_DISTANCE_PROG
                                        width: 94 * main.hScaleFactor
                                        height: 50 * main.vScaleFactor
                                        Layout.alignment: Qt.AlignVCenter | Qt.AlignHCenter
                                        Layout.bottomMargin: 10 * main.vScaleFactor
                                        buttonText: qsTr("4MI") + application.emptyString
                                        selected: valueMatches
                                        onClicked: {
                                            keypad.clear() // Clear the keypad whenever there is an input
                                            root.model.setProperty(index, "dataValue", 4)
                                            root.model.setProperty(index, "units", "MI")
                                            mainProgItem.dataItemChanged()
                                        }
                                    }
                                }
                            }

                            Component { // Bottom component - Distance Quick Stepmill
                                id: bottomcomp_floor_stepmill

                                RowLayout {
                                    id: expandedDetails1
                                    height: 75 * main.vScaleFactor
                                    width: initialWidth

                                    StandardButton {
                                        id: firstFloorButton

                                        property bool valueMatches: (dataValue == 25)

                                        enabled: workoutProgram.programIndex !== WorkoutProgram.STRIDE_DISTANCE_PROG
                                        width: 94 * main.hScaleFactor
                                        height: 50 * main.vScaleFactor
                                        Layout.alignment: Qt.AlignVCenter | Qt.AlignHCenter
                                        Layout.bottomMargin: 10 * main.vScaleFactor
                                        buttonText: qsTr("25") + application.emptyString
                                        selected: valueMatches
                                        onClicked: {
                                            keypad.clear() // Clear the keypad whenever there is an input
                                            root.model.setProperty(index, "dataValue", 25)
                                            root.model.setProperty(index, "units", "Floors")
                                            mainProgItem.dataItemChanged()
                                        }
                                    }

                                    StandardButton {
                                        id: secondFloorButton

                                        property bool valueMatches: (dataValue == 50)

                                        enabled: workoutProgram.programIndex !== WorkoutProgram.STRIDE_DISTANCE_PROG
                                        width: 94 * main.hScaleFactor
                                        height: 50 * main.vScaleFactor
                                        Layout.alignment: Qt.AlignVCenter | Qt.AlignHCenter
                                        Layout.bottomMargin: 10 * main.vScaleFactor
                                        buttonText: qsTr("50") + application.emptyString
                                        selected: valueMatches
                                        onClicked: {
                                            keypad.clear() // Clear the keypad whenever there is an input
                                            root.model.setProperty(index, "dataValue", 50)
                                            root.model.setProperty(index, "units", "Floors")
                                            mainProgItem.dataItemChanged()
                                        }
                                    }

                                    StandardButton {
                                        id: thirdFloorButton

                                        property bool valueMatches: (dataValue == 100)

                                        enabled: workoutProgram.programIndex !== WorkoutProgram.STRIDE_DISTANCE_PROG
                                        width: 94 * main.hScaleFactor
                                        height: 50 * main.vScaleFactor
                                        Layout.alignment: Qt.AlignVCenter | Qt.AlignHCenter
                                        Layout.bottomMargin: 10 * main.vScaleFactor
                                        buttonText: qsTr("100") + application.emptyString
                                        selected: valueMatches
                                        onClicked: {
                                            keypad.clear() // Clear the keypad whenever there is an input
                                            root.model.setProperty(index, "dataValue", 100)
                                            root.model.setProperty(index, "units", "Floors")
                                            mainProgItem.dataItemChanged()
                                        }
                                    }
                                }
                            }

                            Component { // Bottom component - Gender
                                id: bottomComp_gender

                                RowLayout {
                                    id: genderDetails
                                    height: 75 * main.vScaleFactor
                                    width: initialWidth

                                    StandardButton {
                                        id: maleButton
                                        width: 94 * main.hScaleFactor
                                        height: 50 * main.vScaleFactor
                                        Layout.alignment: Qt.AlignVCenter | Qt.AlignHCenter
                                        Layout.bottomMargin: 10 * main.vScaleFactor
                                        buttonText: qsTr("MALE") + application.emptyString
                                        selected: true
                                        onClicked: {
                                            SetupJS.genderSliderValueChanged(model, 0)
                                            selected = true
                                            femaleButton.selected = false
                                        }
                                    }

                                    StandardButton {
                                        id: femaleButton
                                        width: 94 * main.hScaleFactor
                                        height: 50 * main.vScaleFactor
                                        Layout.alignment: Qt.AlignVCenter | Qt.AlignHCenter
                                        Layout.bottomMargin: 10 * main.vScaleFactor
                                        buttonText: qsTr("FEMALE") + application.emptyString
                                        onClicked: {
                                            SetupJS.genderSliderValueChanged(model, 1)
                                            selected = true
                                            maleButton.selected = false
                                        }
                                    }
                                }
                            }

                            Component { // Bottom component - Segment
                                id: bottomcomp_segment

                                ColumnLayout {
                                    height: 107 * main.vScaleFactor
                                    width: initialWidth
                                    spacing: 2 * main.vScaleFactor
                                    LayoutMirroring.enabled: main.rightToLeft
                                    LayoutMirroring.childrenInherit: true

                                    Item {
                                        width: initialWidth
                                        height: calculatedSegTimeValue1.height
                                        anchors.topMargin: 20 * main.vScaleFactor

                                        TextLabel {
                                            id: calculatedSegTime1
                                            text: qsTr("SEGMENT TIME:") + application.emptyString
                                            font.pixelSize: root.paramTextSize
                                            anchors.left: parent.left
                                            anchors.leftMargin: 30 * main.hScaleFactor
                                            anchors.bottom: parent.bottom
                                        }

                                        NumberLabel {
                                            id: calculatedSegTimeValue1
                                            text: Format.formatTimeString(mainProgItem.segTime)
                                            font.pixelSize: 21 * main.vScaleFactor
                                            anchors.right: parent.right
                                            anchors.rightMargin: 30 * main.hScaleFactor
                                        }
                                    }

                                    TextLabel {
                                        id: calculatedSegTimeNote
                                        Layout.alignment: Layout.Left
//                                        anchors.left: parent.left
                                        anchors.leftMargin: 30 * main.hScaleFactor
                                        text: qsTr("MINIMUM SEGMENT TIME IS 1:00") + application.emptyString
                                        width: initialWidth
                                        font.pixelSize: 14 * main.vScaleFactor
                                    }

                                    Item { // slider control box seen after expanding most simple items (age, weight, time, etc.)
                                        id: details
                                        height: 60 * main.vScaleFactor
                                        width: dataElement.initialWidth

                                        property int value: {sliderObject3 === undefined ? sliderObject3.value : 0}

                                        SliderControl {
                                            id: sliderObject3
                                            valueProxy: model.dataValue
                                            Component.onCompleted: {
                                                value = model.dataValue
                                            }
                                            onValueChanged: SetupJS.sliderObject3ValueChanged(model, value)
                                            onInput: keypad.clear() // Clear the keypad whenever there is an input
                                            showValue: false
                                            minValue: model.minValue
                                            maxValue: model.maxValue
                                            value: model.dataValue
                                            precision: model.precision
                                            stepValue: {
                                                if (model.precision === 0) {
                                                    return 1
                                                } else if (model.precision === 1) {
                                                    return 0.1
                                                }
                                            }
                                            anchors.top: parent.top
                                            anchors.horizontalCenter: parent.horizontalCenter
                                            width: 0.75 * parent.width
                                            height: 0.7 * parent.height
                                        }
                                    }
                                }
                            }
                        }
                    }
                }

                Rectangle {
                    id: eGymCardioTestSetup

                    visible: main.eGymCardioTest
                    color: "transparent"
                    radius: main.defaultRadius
                    border.color: main.colorUIBorder
                    border.width: main.widthBorder
                    width: 470 * main.hScaleFactor
                    height: 180 * main.vScaleFactor

                    TextLabel {
                        id: cardioTestLabel
                        anchors {
                            top: parent.top
                            topMargin: 20 * main.vScaleFactor
                            horizontalCenter: parent.horizontalCenter
                        }

                        text: qsTr("SELECT YOUR FITNESS LEVEL")
                        font.pixelSize: 24 * main.vScaleFactor
                    }

                    Row {
                        spacing: 30 * main.hScaleFactor
                        anchors {
                            top: cardioTestLabel.bottom
                            topMargin: 20 * main.vScaleFactor
                            horizontalCenter: parent.horizontalCenter
                        }

                        Repeater {
                            model: [{label: qsTranslate("SetupParams", "NOT FIT") + application.emptyString, icon: "eGym/NotFitIcon"},
                                    {label: qsTranslate("SetupParams", "AVERAGE") + application.emptyString, icon: "eGym/AverageIcon"},
                                    {label: qsTranslate("SetupParams", "VERY FIT") + application.emptyString, icon: "eGym/VeryFitIcon"}]
                            delegate: Component {
                                StandardButton {
                                    width: 100 * main.hScaleFactor
                                    height: 100 * main.vScaleFactor
                                    icon.sourceName: modelData.icon
                                    buttonText: modelData.label
                                    selected: eGym.fitnessLevel === index
                                    onClicked: {
                                        eGym.fitnessLevel = index
                                    }
                                }
                            }
                        }

                    }

                    Component.onCompleted: eGym.fitnessLevel = ""
                }
            }
        }

        Column {
            id: cautionMessage

            property variant item: myListView.model ? myListView.model.get(selectedIndex) : undefined
            property bool showInclineMessage: item !== undefined && isTreadmill && (item.fieldType === WorkoutProgram.INCLINE_TYPE || item.fieldType === WorkoutProgram.MAX_INCLINE_TYPE || (item.fieldType === WorkoutProgram.LEVEL_TYPE && workoutProgram.programIndex !== WorkoutProgram.TREAD_LEG_SHAPER_PROG))
            property bool showWorkloadMessage: item !== undefined && !isTreadmill  && !main.isStepmill && (item.fieldType === WorkoutProgram.LEVEL_TYPE || item.fieldType === WorkoutProgram.WORKLOAD_TYPE)
            property bool showSpeedMessage: item !== undefined && (SetupJS.isSpeedType(item.fieldType) || (item.fieldType === WorkoutProgram.LEVEL_TYPE && workoutProgram.programIndex === WorkoutProgram.TREAD_LEG_SHAPER_PROG))

            visible: (showInclineMessage || showWorkloadMessage || showSpeedMessage) && !isAdjustWorkout
            anchors {
                left: keypad.left
                leftMargin: 40 * main.hScaleFactor
                bottom: parent.bottom
                bottomMargin: 30 * main.vScaleFactor
            }
            width: 204 * main.hScaleFactor
            height: 200 * main.vScaleFactor
            spacing: 10 * main.vScaleFactor

            ScalableImage {
                id: cautionSymbol
                sourceName: "Icon_Warning";
            }

            Text {
                id: cautionTitle
                text: qsTr("CAUTION") + application.emptyString
                color: "orange"
                font.bold: false
                font.pixelSize: 18 * main.vScaleFactor
            }

            Text {
                id: cautionText
                wrapMode: Text.WordWrap
                text: {
                    if (cautionMessage.showInclineMessage) {
                        return qsTr("Use caution when selecting incline level. Injury can result if not prepared for incline levels beyond your physical limits.") + application.emptyString
                    } else if (cautionMessage.showWorkloadMessage) {
                        return qsTr("Use caution when selecting level. Injury can result if not prepared for workload resistance beyond your physical limits.") + application.emptyString
                    } else if (cautionMessage.showSpeedMessage) {
                        return qsTr("Use caution when selecting speed. Injury can result if not prepared for speeds beyond your physical limits.") + application.emptyString
                    } else {
                        return ""
                    }
                }
                color: main.colorFontLight
                font.pixelSize: 14 * main.vScaleFactor
                font.bold: false
                width: parent.width
            }
        }
    }
}
