import QtQuick 2.12
import Components 1.0
import "views.js" as TheViews

Item {
    id: root

    property variant workoutScreen
    property alias contentAreaHeight: contentArea.height
    property alias contentAreaWidth: contentArea.width

    property bool isToolbarVisible: true

    property real totalWidth: main.screenWidth * 1734 / 1920

    property real viewAreaWidth: totalWidth
    property real viewAreaHeight: main.screenHeight * 743 / 1080
    property real toolbarWidth: main.screenWidth * 139 / 1920
    property real toolbarHeight: viewAreaHeight

    property variant viewGroup
    property string defaultView: ""
    property int defaultIndex: 0
    property string currentViewName: ""
    property bool isMoving: contentArea.moving || _clickedButtonMoving
    property int _clickedButtonIndex
    property bool _clickedButtonMoving: false
    property int _oldIndex: -1
    property int _viewCreationIndex: 0
    property int _creating: viewCreateTimer.running
    property int currentIndex: 0
    property bool persistMenu: false

    signal currentViewChanged(string view, bool isRunState)
    signal viewsCompleted()
    signal viewsDeinit()

    anchors.horizontalCenter: parent.horizontalCenter
    z: 99

    states: [
        State {
            name: "toolbarHidden"
            when: !isToolbarVisible
            PropertyChanges { target: buttonBarBackground; opacity: 0 }
            PropertyChanges { target: buttonBar; opacity: 0 }
        },
        State {
            name: "toolbarShown"
            when: isToolbarVisible
            PropertyChanges { target: buttonBarBackground; opacity: 1.0 }
            PropertyChanges { target: buttonBar; opacity: 1.0 }
        }
    ]

    onStateChanged: {
        if(state === "toolbarShown" && !options.navBarAlwaysOn) {
            hideToolbarTimer.start()
        }
    }

    function reset() {
        isToolbarVisible = true;
        if(!options.navBarAlwaysOn)
            hideToolbarTimer.start()
    }

    function handleToolbarDisplay()
    {
        if(isToolbarVisible === true && !root.persistMenu && !options.navBarAlwaysOn) {
            hideToolbarTimer.restart()
        } else {
            isToolbarVisible = true
        }
    }

    Timer {
        id: hideToolbarTimer
        interval: 5000
        running: false
        repeat: false
        onTriggered: {
            var isMoving = false
            if(contentArea.currentItem){
                var obj = contentArea.currentItem.loadedItem
                if(obj && obj.hasOwnProperty("isMenuMoving")) {
                    isMoving = obj.isMenuMoving
                }
            }
            /* Extend timer another 5 seconds if menu is still moving */
            if(isMoving === true) {
                handleToolbarDisplay()
            } else if(!root.persistMenu){
                isToolbarVisible = false
            }
        }
    }

    onCurrentIndexChanged: {
        console.debug("ViewContainer: " + currentIndex)
        contentArea.currentIndex = currentIndex
    }

    function destroyViews() {
        for(var i=1 ; i <= viewListModel.count; i++) {
            var item = viewListModel.get(i);
            var obj = contentArea.children[i].children[0];

            if(obj) {
                console.log("ViewContainer: Destroying", obj);

                if(obj.item && obj.item.hasOwnProperty("viewdestruct")) {
                    obj.item.viewdestruct();
                }

                obj.destroy();
                delete obj;
            }
        }

        viewListModel.clear();
        root._oldIndex = -1;
        currentIndex = -1;
    }

    function setViews(viewgroup) {
        var i;
        root.viewsDeinit(); // signal to the views that they are about to be destroyed
        destroyViews();
        root.viewGroup = viewgroup;

        // Create a dummy set of all views with only the button, EXCEPT the default view
        // This allows us to create the default view before the others
        for (var i = 0; i < viewgroup.views.length; i++) {
            var qml = root.viewGroup.views[i].theqml;
            var button = root.viewGroup.views[i].thebuttons;

            if (button.name === root.defaultView) {
                viewListModel.append({"theqml": qml, "thebuttons": button});
            } else {
                viewListModel.append({"theqml": "", "thebuttons": button});
            }
        }

        // begin the creation of views on at a time in a timer, to avoid blocking for a long period of time
        root._viewCreationIndex = 0;
        viewCreateTimer.start();
    }

    // create and display a single view from the viewGroup model
    function createView(index) {
        var qml = root.viewGroup.views[index].theqml;
        var button = root.viewGroup.views[index].thebuttons;
        viewListModel.setProperty(index, "theqml", qml);
    }

    function findViewIndex(viewName) {
        for (var i = 0; i < viewListModel.count; i++) {
            if (viewListModel.get(i).thebuttons.name === viewName) {
                return i;
            }
        }

        return -1;
    }

    // viewCreateTimer loads one qml view from the viewGrop upon each timer expiration, until reaching end of array
    Timer {
        id: viewCreateTimer
        interval: 1
        repeat: true
        onTriggered: {
            createView(root._viewCreationIndex++)

            if(root._viewCreationIndex >= viewGroup.views.length) {
                stop()
                root.viewsCompleted() // signal that the views have been constructed
            }
        }
    }

    Timer {
        id: contentAreaTransitionTimer
        interval: contentArea.highlightMoveDuration
        running: false
        repeat: false
        onTriggered: {
            contentAreaTransitionTimer.stop()
            _clickedButtonMoving  = false
        }
    }

    Rectangle {
        id: viewAndToolbarArea
        height: root.viewAreaHeight
        width: root.totalWidth
        anchors.horizontalCenter: root.horizontalCenter
        anchors.verticalCenter: root.verticalCenter
        color: "transparent"
        border.color: main.colorUIBorder
        border.width: main.widthBorder
    }

    ListModel {
        id: viewListModel
    }

    Component {
        id:viewDelegate

        Item {
            id: wrapper
            width: parent.width
            height: parent.height

            property alias loadedItem: loader.item

            Loader {
                id: loader
                source: theqml
                anchors.fill: parent
                onStatusChanged: {
                    if (loader.status === Loader.Ready) {
                        // If this is the default view, scroll to it
                        if (thebuttons.name === root.defaultView) {
                            currentIndex = index
                            _oldIndex = index // set to allow calling hidden() method for this view when changed

                            // call the shown() method for new view, if defined
                            if(item && item.hasOwnProperty("shown")) {
                                item.shown()
                            }

                            if(item && item.hasOwnProperty("isMenuDisplayed")) {
                                item.isMenuDisplayed = isToolbarVisible
                            }
                        }
                    }
                }
            }

            Connections {
                target: loader.item
                onAction: handleToolbarDisplay()
                onIsMenuMovingChanged: {
                    if(loader.item.isMenuMoving === false) {
                        handleToolbarDisplay()
                    }
                }
            }
        }
    }

    onIsToolbarVisibleChanged: {
        if(contentArea.currentItem)
        {
            var obj = contentArea.currentItem.loadedItem
            if(obj && obj.hasOwnProperty("isMenuDisplayed")) {
                obj.isMenuDisplayed = isToolbarVisible
            }
        }
    }

    PathView {
        id: contentArea
        width: root.viewAreaWidth
        height: root.viewAreaHeight
        anchors.bottom: viewAndToolbarArea.bottom
        anchors.left: viewAndToolbarArea.left
        model:viewListModel
        delegate: viewDelegate
        highlightRangeMode: PathView.StrictlyEnforceRange
        highlightMoveDuration: 0
        clip: true
        visible: true
        interactive: false
        onCurrentIndexChanged: {
            if(model.count < 1) {
                return
            }

            var view_name = model.get(currentIndex).thebuttons.name
            root.currentViewName = view_name
            print("ViewContainer: View changed = " + view_name)

            // call the hiden() method for old view, if defined
            if(root._oldIndex >=0 && root._oldIndex < model.count) {
                var oldobj = contentArea.children[root._oldIndex + 1].loadedItem

                if(oldobj && oldobj.hasOwnProperty("hidden")) {
                    oldobj.hidden()
                    root.handleToolbarDisplay()
                }
            }

            root._oldIndex = currentIndex

            if(contentArea.currentItem)
            {
                var obj = contentArea.currentItem.loadedItem
                if(obj && obj.hasOwnProperty("shown")) {
                    obj.shown()
                }

                if(obj && obj.hasOwnProperty("isMenuDisplayed")) {
                    obj.isMenuDisplayed = isToolbarVisible
                }
            }
        }
        preferredHighlightBegin: 0.5
        preferredHighlightEnd: 0.5
        path: Path {
            startX: contentArea.width * (1 - contentArea.model.count) / 2
            startY: contentArea.height / 2

            PathLine {
                x: contentArea.width * (1 + contentArea.model.count) / 2
                y: contentArea.height / 2
            }
        }
    }

    Rectangle{
        id: buttonBarBackground

        color: main.colorToolbarBackground
        width: root.toolbarWidth
        anchors.bottom: viewAndToolbarArea.bottom
        anchors.top: viewAndToolbarArea.top
        anchors.right: viewAndToolbarArea.right
        anchors.margins: main.widthBorder
    }

    ListView {
        id: buttonBar
        anchors.fill: buttonBarBackground
        anchors.topMargin: spacing
        anchors.verticalCenter: buttonBarBackground.verticalCenter
        model:viewListModel
        delegate: buttonBarDelegate
        orientation: ListView.Vertical
        interactive: false
        spacing: 15 * main.vScaleFactor

        Component {
            id: buttonBarDelegate

            Item {
                id: buttonItem

                height: 70 * main.vScaleFactor //root.toolbarHeight / buttonBar.count
                width: height
                anchors.horizontalCenter: parent.horizontalCenter

                Component.onCompleted: {
                    root.currentIndex = root.defaultIndex;
                }

                Rectangle {
                    id: buttonBorder
                    color: "transparent"
                    anchors.fill: buttonItem
                    radius: main.defaultRadius
                    border.color: main.colorUIBorder
                    border.width: main.widthBorder
                    visible: root.currentIndex == index ? true : false
                    z: icon.z + 1
                }

                ScalableImage {
                    id: icon
                    property bool selected: index == root.currentIndex
                    fillMode: Image.PreserveAspectFit
                    anchors.fill: buttonBorder
//                    anchors.margins: main.widthBorder
                    sourceSize.width: parent.width
                    sourceSize.height: parent.height
                    antialiasing: true
                    sourceName: thebuttons ? selected ? thebuttons.icon_image_down : thebuttons.icon_image : ""
                    MouseArea {
                        anchors.fill: parent
                        enabled: root.isToolbarVisible
                        onClicked: {
                            /* restart 5 second timer when view is selected */
                            handleToolbarDisplay()
                            root.currentIndex = index
                            buzzer.buttonPress()
                        }

                        Rectangle {
                            anchors.fill: parent
                            color: "red"
                            opacity: 0.3
                            visible: main.showStimulusArea
                        }
                    }
                }
            }
        }
    }

    ViewContainerVolumeControls {
        id: volume_controls

        /**
         * @brief type:bool Property that determines if the volume controls are shown.
         */
//        property bool canShow: main.state === "RunState" || ipod_window.visible

        z: 200
        opacity: audioManager.audioPlaying ? 1.0 : 0.0
        enabled: (visible && opacity > 0.0)
        visible: isToolbarVisible && root.currentViewName !== "TV_VIEW" && root.currentViewName !== "WEB_VIEW" && (browserMute.muted === false || bluetoothDeviceAudioWatcher.available === true)
        anchors.bottom: buttonBar.bottom
        anchors.bottomMargin: 0 * main.vScaleFactor
        anchors.right: buttonBar.left
        anchors.rightMargin: 0 * main.hScaleFactor
        config: root.currentViewName === "PROFILE_VIEW" ? "line" : "lshape"

        // synchronize the control display with actual volume if changed elsewhere
        Connections {
            target: audioManager
            onVolumeChanged: {
                if(audioManager.volume != volume_controls.percent) {
                    volume_controls.vol_setting = audioManager.volume * (volume_controls.vol_max - volume_controls.vol_min) / 100;
                }
            }
            onMuteChanged: volume_controls.muted = audioManager.mute
        }

        /*onBluetoothHeadsetButtonClicked: {
            if(bluetoothAdapter.connectedHeadset === true) {
                bluetoothAdapter.disconnectHeadset();
            } else {
                btDeviceSelectionDialog.show();
                btDeviceSelectionDialog.state = "list";
            }
            buzzer.buttonPress()
        }*/
        onMutedChanged: audioManager.setMute(muted)
        onPercentChanged: audioManager.setVolume(percent)
/*        onSourceButtonClicked: {
            print("Audio Source Button Pressed.");
            audioManager.nextSource();
            buzzer.buttonPress()
        }
        showSourceButton: audioManager.audioSources.length > 1 && !bluetoothAdapter.connectedHeadset
*/
        onAction: handleToolbarDisplay()

        Behavior on opacity { NumberAnimation { duration: 200; easing.type: Easing.InOutQuad } }
    }
}
